# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
#                     Edward Fauchon-Jones <edward.fauchon-jones@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import pesummary
from pesummary.core.webpage import tables
from pesummary.core.webpage.base import Base

import sys
from pygments import highlight
from pygments.lexers import get_lexer_by_name
from pygments.formatters import HtmlFormatter
import time

BOOTSTRAP = """<!DOCTYPE html>
<html lang='en'>
    <title>title</title>
    <meta charset='utf-8'>
    <meta name='viewport' content='width=device-width, initial-scale=1'>
    <link rel='stylesheet' href='https://maxcdn.bootstrapcdn.com/bootstrap/4.1.3/css/bootstrap.min.css'>
    stylesheet elements
</head>
<body style='background-color:#F8F8F8; margin-top:5em'>
"""

HOME_SCRIPTS = """    <script src='https://ajax.googleapis.com/ajax/libs/jquery/3.3.1/jquery.min.js'></script>
    <script src='https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.14.3/umd/popper.min.js'></script>
    <script src='https://maxcdn.bootstrapcdn.com/bootstrap/4.1.3/js/bootstrap.min.js'></script>
    <script src='./js/combine_corner.js'></script>
    <script src='./js/grab.js'></script>
    <script src='./js/modal.js'></script>
    <script src='./js/multi_dropbar.js'></script>
    <script src='./js/multiple_posteriors.js'></script>
    <script src='./js/search.js'></script>
    <script src='./js/side_bar.js'></script>
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.7.0/css/font-awesome.min.css">
    <link rel="stylesheet" href="./css/navbar.css">
    <link rel="stylesheet" href="./css/font.css">
    <link rel="stylesheet" href="./css/table.css">
"""

OTHER_SCRIPTS = """    <script src='https://ajax.googleapis.com/ajax/libs/jquery/3.3.1/jquery.min.js'></script>
    <script src='https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.14.3/umd/popper.min.js'></script>
    <script src='https://maxcdn.bootstrapcdn.com/bootstrap/4.1.3/js/bootstrap.min.js'></script>
    <script src='../js/combine_corner.js'></script>
    <script src='../js/grab.js'></script>
    <script src='../js/modal.js'></script>
    <script src='../js/multi_dropbar.js'></script>
    <script src='../js/multiple_posteriors.js'></script>
    <script src='../js/search.js'></script>
    <script src='../js/side_bar.js'></script>
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.7.0/css/font-awesome.min.css">
    <link rel="stylesheet" href="../css/navbar.css">
    <link rel="stylesheet" href="../css/font.css">
    <link rel="stylesheet" href="../css/table.css">
"""


def make_html(web_dir, title="Summary Pages", pages=None, stylesheets=[],
              label=None):
    """Make the initial html page.

    Parameters
    ----------
    web_dir: str
        path to the location where you would like the html file to be saved
    title: str, optional
        header title of html page
    pages: list, optional
        list of pages that you would like to be created
    stylesheets: list, optional
        list of stylesheets to including in the html page. It is assumed all
        stylesheets are located in the `css` in the root of the summary pages.
        This should be provided as basenames without extension that is assumed
        to be `.css`.
    """
    for i in pages:
        if i != "home":
            f = open("{}/html/{}.html".format(web_dir, i), "w")
        else:
            f = open("{}/{}.html".format(web_dir, i), "w")
        stylesheet_elements = ''.join([
            "  <link rel='stylesheet' href='../css/{0:s}.css'>\n".format(s)
            for s in stylesheets])
        bootstrap = BOOTSTRAP.split("\n")
        bootstrap[1] = "  <title>{}</title>".format(title)
        bootstrap[-4] = stylesheet_elements
        bootstrap = [j + "\n" for j in bootstrap]
        f.writelines(bootstrap)
        if i != "home":
            scripts = OTHER_SCRIPTS.split("\n")
        else:
            scripts = HOME_SCRIPTS.split("\n")
        scripts = [j + "\n" for j in scripts]
        f.writelines(scripts)


def open_html(web_dir, base_url, html_page, label=None):
    """Open html page ready so you can manipulate the contents

    Parameters
    ----------
    web_dir: str
        path to the location where you would like the html file to be saved
    base_url: str
        url to the location where you would like the html file to be saved
    page: str
        name of the html page that you would like to edit
    label: str
        the label that prepends your page name
    """
    try:
        if html_page[-5:] == ".html":
            html_page = html_page[:-5]
    except Exception:
        pass
    if html_page == "home.html" or html_page == "home":
        f = open(web_dir + "/home.html", "a")
    else:
        if label is not None:
            f = open(web_dir + "/html/{}_".format(label) + html_page
                     + ".html", "a")
        else:
            f = open(web_dir + "/html/" + html_page + ".html", "a")
    return page(f, web_dir, base_url, label)


class page(Base):
    """Class to generate and manipulate an html page.
    """
    def __init__(self, html_file, web_dir, base_url, label):
        self.html_file = html_file
        self.web_dir = web_dir
        self.base_url = base_url
        self.label = label
        self.content = []

    def _header(self, approximant):
        """
        """
        self.add_content("<h7 hidden>{}</h7>".format(self.label))
        self.add_content("<h7 hidden>{}</h7>".format(approximant))

    def _footer(self, user, rundir):
        """
        """
        command = ""
        for i in sys.argv:
            command += " "
            if i[0] == "-":
                command += "\n"
            command += "{}".format(i)
        self.make_div(
            _class='jumbotron', _style='margin-bottom:0; line-height: 0.5;'
            + 'background-color:#E0E0E0')
        self.add_content(
            "<p style='font-family:Arial-body'>This page was produced by {} at "
            "{} on {} on behalf of the Parameter Estimation group\n".format(
                user, time.strftime("%H:%M"), time.strftime("%B %d %Y")), indent=2)
        self.add_content("<p style='font-family:Arial-body'>Run directories "
                         "found at {}</p>\n".format(rundir), indent=2)
        self.add_content("<p style='font-family:Arial-body'>This code was "
                         "generated with the following command line call:</p>", indent=2)
        self.add_content("<p> </p>", indent=2)
        self.make_div(
            _class='container', _style='background-color:#FFFFFF; '
            'box-shadow: 0 0 5px grey; line-height: 1.5')
        styles = self.make_code_block(language='shell', contents=command)
        with open('{0:s}/css/command_line.css'.format(self.web_dir), 'w') as g:
            g.write(styles)
        self.end_div()
        self.make_div(_style="text-align:center")
        self.add_content(
            "<a href='https://git.ligo.org/lscsoft/pesummary'>"
            "View PESummary v%s on git.ligo.org</a> | "
            "<a href='https://git.ligo.org/lscsoft/pesummary/issues'>"
            "Report an issue</a> | <a href='https://docs.ligo.org/lscsoft/"
            "pesummary/summarypage.html'> Help on using this webpage</a>" % (
                pesummary.__version__), indent=2)
        self.end_div()
        self.end_div()

    def _setup_navbar(self, background_colour):
        if background_colour == "navbar-dark" or background_colour is None:
            self.add_content("<nav class='navbar navbar-expand-sm navbar-dark "
                             "bg-dark fixed-top'>\n")
        else:
            self.add_content("<nav class='navbar navbar-expand-sm fixed-top "
                             "navbar-custom' style='background-color: %s'>" % (
                                 background_colour))
        self.add_content("<a class='navbar-brand' href='#' style='color: white'"
                         ">Navbar</a>\n", indent=2)
        self.add_content("<button class='navbar-toggler' type='button' "
                         "data-toggle='collapse' data-target='#collapsibleNavbar'>\n", indent=2)
        self.add_content("<span class='navbar-toggler-icon'></span>\n", indent=4)
        self.add_content("</button>\n", indent=2)

    def make_header(self, approximant="IMRPhenomPv2"):
        """Make header for document in bootstrap format.

        Parameters
        ----------
        title: str, optional
            header title of html page
        approximant: str, optional
            the approximant that you are analysing
        """
        self._header(approximant)

    def make_footer(self, user=None, rundir=None):
        """Make footer for document in bootstrap format.
        """
        self._footer(user, rundir)

    def make_banner(self, approximant=None, key="Summary", _style=None):
        """Make a banner for the document.
        """
        self.make_div(indent=2, _class='banner', _style=_style)
        self.add_content("%s" % (approximant))
        self.end_div()
        self.make_div(indent=2, _class='paragraph')
        if key == "Summary":
            self.add_content(
                "The figures below show the summary plots for the run")
        elif key == "config":
            self.add_content(
                "Below is the config file for %s" % (approximant))
        elif key == "corner":
            self.add_content(
                "Below is the custom corner plotter for %s" % (approximant))
        elif key == "Comparison":
            self.add_content(
                "Below are the summary comparison plots")
        elif key == "Version":
            self.add_content(
                "Below is the version information for all files passed and the "
                "PESummary version used to generate these pages")
        elif key == "Publication":
            self.add_content(
                "Below are publication quality plots for the passed result "
                "files")
        elif key == "Logging":
            self.add_content("Below is the output from the PESummary code")
        elif key == "Notes":
            self.add_content("Below are your custom notes")
        elif key == "classification":
            self.add_content(
                "Below we look at the source probabilities and plots for the "
                "passed result file")
        elif key == "sampler_kwargs":
            self.add_content(
                "Sampler information extracted from the result file")
        elif key == "meta_data":
            self.add_content(
                "Meta data extracted from the result file")
        else:
            self.add_content(
                "The figures below show the plots for %s" % (approximant))
        self.end_div()

    def make_navbar(self, links=None, samples_path="./samples", search=True,
                    histogram_download=None,
                    background_color="navbar-dark",
                    hdf5=False):
        """Make a navigation bar in boostrap format.

        Parameters
        ----------
        links: list, optional
            list giving links that you want your navbar to include. If a
            dropdown option is required, give a 2d list showing the main link
            followed by dropdown links. For instance, if you wanted to have
            links corner plots and a dropdown link named 1d_histograms with
            options mass1, mass2, mchirp, then we would give,

                links=[corner, [1d_histograms, [mass1, mass2, mchirp]]]

        samples_path: str, optional
            path to the location of the meta file
        search: bool, optional
            if True, search bar will be given in navbar
        histogram_download: str, optional
            path to the location of the data associated with the histogram
        hdf5: Bool, optional
            true if a hdf5 file format is chosen for the meta file
        """
        self._setup_navbar(background_color)
        if links is None:
            raise Exception("Please specify links for use with navbar\n")
        self.add_content("<div class='collapse navbar-collapse' id='collapsibleNavbar'>\n", indent=4)
        self.add_content("<ul class='navbar-nav'>\n", indent=6)
        for i in links:
            if type(i) == list:
                self.add_content("<li class='nav-item'>\n", indent=8)
                self.add_content("<li class='nav-item dropdown'>\n", indent=10)
                self.add_content("<a class='nav-link dropdown-toggle' "
                                 "href='#' id='navbarDropdown' role='button' "
                                 "data-toggle='dropdown' aria-haspopup='true' "
                                 "aria-expanded='false'>\n", indent=12)
                self.add_content("{}\n".format(i[0]), indent=12)
                self.add_content("</a>\n", indent=12)
                self.add_content("<ul class='dropdown-menu' aria-labelledby='dropdown1'>\n", indent=12)
                for j in i:
                    if type(j) == list:
                        if len(j) > 1:
                            if type(j[1]) == list:
                                self.add_content("<li class='dropdown-item dropdown'>\n", indent=14)
                                self.add_content("<a class='dropdown-toggle' id='{}' "
                                                 "data-toggle='dropdown' "
                                                 "aria-haspopup='true' "
                                                 "aria-expanded='false'>{}</a>\n".format(j[0], j[0]), indent=16)
                                self.add_content("<ul class='dropdown-menu' "
                                                 "aria-labelledby='{}'>\n".format(j[0]), indent=16)
                                for k in j[1]:
                                    if type(k) == dict:
                                        key = list(k.keys())[0]
                                        self.add_content(
                                            "<li class='dropdown-item' href='#' "
                                            "onclick='grab_html(\"{}\", label=\"{}\")'>"
                                            "<a>{}</a></li>\n".format(key, k[key], key), indent=18)
                                    else:
                                        self.add_content(
                                            "<li class='dropdown-item' href='#' "
                                            "onclick='grab_html(\"{}\")'>"
                                            "<a>{}</a></li>\n".format(k, k), indent=18)

                                self.add_content("</ul>", indent=16)
                                self.add_content("</li>", indent=14)
                            else:
                                for k in j:
                                    if type(k) == dict:
                                        key = list(k.keys())[0]
                                        self.add_content(
                                            "<li class='dropdown-item' href='#' "
                                            "onclick='grab_html(\"{}\", label=\"{}\")'>"
                                            "<a>{}</a></li>\n".format(key, k[key], key), indent=14)

                                    else:
                                        self.add_content(
                                            "<li class='dropdown-item' href='#' "
                                            "onclick='grab_html(\"{}\")'>"
                                            "<a>{}</a></li>\n".format(k, k), indent=14)

                        else:
                            if type(j[0]) == dict:
                                key = list(j[0].keys())[0]
                                self.add_content(
                                    "<li class='dropdown-item' href='#' "
                                    "onclick='grab_html(\"{}\", label=\"{}\")'>"
                                    "<a>{}</a></li>\n".format(key, j[0][key], key), indent=14)

                            else:
                                self.add_content(
                                    "<li class='dropdown-item' href='#' "
                                    "onclick='grab_html(\"{}\")'>"
                                    "<a>{}</a></li>\n".format(j[0], j[0]), indent=14)

                self.add_content("</ul>\n", indent=12)
                self.add_content("</li>\n", indent=10)
            else:
                self.add_content("<li class='nav-item'>\n", indent=8)
                if i == "home":
                    self.add_content("<a class='nav-link' "
                                     "href='#' onclick='grab_html(\"{}\")'"
                                     ">{}</a>\n".format(i, i), indent=10)
                else:
                    if type(i) == dict:
                        key = list(i.keys())[0]
                        self.add_content(
                            "<a class='nav-link' "
                            "href='#' onclick='grab_html(\"{}\", label=\"{}\")'"
                            ">{}</a>\n".format(key, i[key], key), indent=10)

                    else:
                        self.add_content(
                            "<a class='nav-link' "
                            "href='#' onclick='grab_html(\"{}\")'"
                            ">{}</a>\n".format(i, i), indent=10)

                self.add_content("</li>\n", indent=8)
        self.add_content("</ul>\n", indent=6)
        self.add_content("</div>\n", indent=4)
        if histogram_download:
            self.add_content("<a href='%s' download>" % (histogram_download),
                             indent=4)
            self.add_content(
                "<button type='submit' style='margin-right: 15px; cursor:pointer'> "
                "<i class='fa fa-download'></i> Histogram Data</button>", indent=6)
            self.add_content("</a>", indent=4)
        if hdf5:
            path = '%s/posterior_samples.h5' % (samples_path)
        else:
            path = '%s/posterior_samples.json' % (samples_path)
        self.add_content("<a href='%s' download>" % (path), indent=4)
        self.add_content(
            "<button type='submit' style='margin-right: 15px; cursor:pointer'> "
            "<i class='fa fa-download'></i> Results File</button>", indent=6)
        self.add_content("</a>", indent=4)
        if search:
            self.add_content("<input type='text' placeholder='search' id='search'>\n", indent=4)
            self.add_content("<button type='submit' onclick='myFunction()'>Search</button>\n", indent=4)
        self.add_content("</nav>\n")

    def make_table(self, headings=None, contents=None, heading_span=1,
                   colors=None, accordian_header="Summary Table",
                   accordian=True, format="table-striped table-sm", **kwargs):
        """Generate a table in bootstrap format.

        Parameters
        ----------
        headings: list, optional
            list of headings
        contents: list, optional
            nd list giving the contents of the table.
        heading_span: int, optional
            width of the header cell. By default it will span a single column
        colors: list, optional
            list of colors for the table columns
        """
        if accordian:
            label = accordian_header.replace(" ", "_")
            self.make_container(style=kwargs.get("style", None))
            self.make_div(indent=2, _class='row justify-content-center')
            self.make_div(
                indent=4, _class='accordian', _style='width: 100%',
                _id='accordian%s' % (label))
            self.make_div(indent=6, _class='card')
            self.make_div(
                indent=8, _class='card-header', _style='background-color: #E0E0E0',
                _id='table')
            self.add_content("<h5 class='mb-0'>", indent=10)
            self.make_div(indent=12, _class='row justify-content-center')
            self.add_content(
                "<button class='btn btn-link collapsed' type='button' "
                "data-toggle='collapse' data-target='#collapsetable%s' "
                "aria-expanded='false' aria-controls='collapsetable'>" % (label), indent=14)
            self.add_content(accordian_header)
            self.add_content("</button>")
            self.end_div(indent=12)
            self.end_div(indent=10)
            self.add_content(
                "<div id='collapsetable%s' class='collapse' "
                "aria-labelledby='table' data-parent='#accordian%s'>" % (label, label),
                indent=12)
            self.make_div(_class='card-body', indent=14)
            self.make_div(_class='row justify-content-center', indent=16)
        self.make_div(_class='container', indent=18)
        if type(contents) == list:
            self.make_div(indent=20, _class='table-responsive')
            if heading_span > 1:
                self.add_content("<table class='table table-sm'>\n", indent=22)
            else:
                self.add_content("<table class='table %s'>\n" % (format), indent=24)
            self.add_content("<thead>\n", indent=26)
            self.add_content("<tr>\n", indent=28)
            for i in headings:
                self.add_content("<th colspan='{}'>{}</th>\n".format(heading_span, i), indent=30)
            self.add_content("</tr>\n", indent=28)
            self.add_content("<tbody>\n", indent=26)

            for num, i in enumerate(contents):
                self.add_content("<tr>\n", indent=28)
                if heading_span == 2:
                    for j, col in zip(i, ["#ffffff"] + colors * (len(i) - 1)):
                        self.add_content(
                            "<td style='background-color: {}'>{}</td>\n".format(col, j), indent=30)
                    self.add_content("</tr>", indent=28)
                else:
                    for j in i:
                        self.add_content("<td>{}</td>\n".format(j), indent=30)
                    self.add_content("</tr>\n", indent=28)
            self.add_content("</tbody>\n", indent=26)
            self.add_content("</table>\n", indent=24)
            self.end_div(indent=20)
        elif type(contents) == dict:
            self.make_div(indent=20, _class='table-responsive')
            if heading_span > 1:
                self.add_content("<table class='table table-sm'>\n", indent=22)
            else:
                self.add_content("<table class='table table-striped table-sm'>\n", indent=24)
            self.add_content("<thead>\n", indent=26)
            for j in contents.keys():
                self.add_content("<tr bgcolor='#F0F0F0'>\n", indent=28)
                self.add_content(
                    "<th colspan='{}' class='text-center'>"
                    "{}</th>\n".format(len(contents[j][0]), j), indent=30)
                self.add_content("</tr>\n", indent=28)
                self.add_content("<tr>\n", indent=28)
                for i in headings:
                    self.add_content(
                        "<th colspan='{}' class='text-center'>"
                        "{}</th>\n".format(heading_span, i), indent=30)
                self.add_content("</tr>\n", indent=28)

                for num, i in enumerate(contents[j]):
                    self.add_content("<tr>\n", indent=28)
                    for j in i:
                        self.add_content("<td>{}</td>\n".format(j), indent=30)
                    self.add_content("</tr>\n", indent=28)
                self.add_content("</tbody>\n", indent=26)
            self.add_content("</table>\n", indent=24)
            self.end_div(indent=20)

        self.end_div(indent=18)
        if accordian:
            self.end_div(indent=16)
            self.end_div(indent=14)
            self.end_div(indent=12)
            self.end_div(indent=10)
            self.end_div(indent=8)
            self.end_div(indent=6)
            self.end_div(indent=4)
            self.end_div(indent=2)
            self.end_div()

    def make_code_block(self, language=None, contents=None):
        """Generate a code block hightlighted using pigments.

        Parameters
        ----------
        language: str, optional
            The lanaguge of the configuration file to use for syntax
            highlighting.
        contents: str, optional
            String containing the contents of the config file.

        Returns
        -------
        style: str
            The styles used to highlight the rendered contents.
        """
        lexer = get_lexer_by_name(language)
        formatter = HtmlFormatter(style='manni')
        render = highlight(contents, lexer, formatter)
        self.add_content(render)
        styles = formatter.get_style_defs('.highlight')
        styles += ".highlight {margin: 5px; padding: 10px; background: #FFFFFF}"
        return styles

    def make_table_of_images(self, contents=None, rows=None, columns=None,
                             code="modal", cli=None):
        """Generate a table of images in bootstrap format.

        Parameters
        ----------
        headings: list, optional
            list of headings
        contents: list, optional
            nd list giving the contents of the table.
        carousel: bool, optional
            if True, the images will be configured to work operate as part of
            a carousel
        width: float, optional
            width of the images in the table
        container: bool, optional
            if True, the table of images is placed inside a container
        """
        table = tables.table_of_images(contents, rows, columns, self.html_file,
                                       code=code, cli=cli)
        table.make()

    def insert_image(self, path, justify="center", code=None):
        """Generate an image in bootstrap format.

        Parameters
        ----------
        path: str, optional
            path to the image that you would like inserted
        justify: str, optional
            justifies the image to either the left, right or center
        """
        self.make_container()
        _id = path.split("/")[-1][:-4]
        string = "<img src='{}' id='{}' alt='No image available' ".format(path, _id) + \
                 "style='align-items:center; width:850px; cursor: pointer'"
        if justify == "center":
            string += " class='mx-auto d-block'"
        elif justify == "left":
            string = string[:-1] + " float:left;'"
        elif justify == "right":
            string = string[:-1] + " float:right;'"
        if code:
            string += " onclick='{}(\"{}\")'".format(code, _id)
        string += ">\n"
        self.add_content(string, indent=2)
        self.end_container()

    def make_accordian(self, headings=None, content=None):
        """Generate an accordian in bootstrap format with images as content.

        Parameters
        ----------
        headings: list, optional
            list of headings that you want your accordian to have
        content: nd list, optional
            n dimensional list where n is the number of rows. The content
            of each list should be the path to the location of the image
        """
        self.make_div(_class='row justify-content-center')
        self.add_content("<div class='accordian' id='accordian' style='width:70%'>\n", indent=2)
        for num, i in enumerate(headings):
            self.add_content("<div class='card' style='border: 0px solid black'>\n", indent=4)
            self.add_content("<div class='card-header' id='{}'>\n".format(i), indent=6)
            self.add_content("<h5 class='mb-0'>\n", indent=8)
            self.add_content(
                "<button class='btn btn-link collapsed' type='button' data-toggle='collapse' "
                "data-target='#collapse{}' aria-expanded='false' ".format(i)
                + "aria-controls='collapse{}'>\n".format(i), indent=10)
            self.add_content("{}\n".format(i), indent=12)
            self.add_content("</button>\n", indent=10)
            self.add_content("</h5>\n", indent=8)
            self.add_content("</div>\n", indent=6)
            self.add_content(
                "<div id='collapse{}' class='collapse' ".format(i)
                + "aria-labelledby='{}' data-parent='#accordian'>\n".format(i), indent=6)
            self.add_content("<div class='card-body'>\n", indent=8)
            self.add_content("<img src='{}' ".format(content[num])
                             + "alt='No image available' style='width:700px;' "
                             + "class='mx-auto d-block'>\n", indent=10)
            self.add_content("</div>\n", indent=8)
            self.add_content("</div>\n", indent=6)
            self.add_content("</div>\n", indent=4)
        self.add_content("</div>\n", indent=2)
        self.add_content("</div>\n")

    def make_search_bar(self, sidebar=None, popular_options=None, label=None,
                        code="combine"):
        """Generate a search bar to combine the corner plots
        javascript.

        Parameters
        ----------
        sidebar: list, optional
            a list of parameters that you would like included in the side bar
        popular_options: list, optional
            a list of popular options for your search bar
        """
        ids = "canvas" if code == "combine" else code
        self.add_content("<link rel='stylesheet' href='../css/side_bar.css'>\n")
        self.add_content("<div class='w3-sidebar w3-bar-block w3-border-right sidenav' "
                         "style='display:none' id='mySidebar'>\n")
        self.add_content("<button onclick='side_bar_close()' class='close'>&times;</button>\n", indent=2)
        if sidebar:
            for i in sidebar:
                self.add_content("<input type='checkbox' name='type' "
                                 "value='{}' id='{}' style='text-align: center; margin: 0 5px 0;'"
                                 ">{}<br>\n".format(i, i, i, i), indent=2)
        self.add_content("</div>")
        self.add_content("<div class='row justify-content-center'>")
        self.add_content(
            "<p style='margin-top:2.5em; font-family: Arial-body; font-size:14px'>"
            " Input the parameter names that you would like to compare</p>", indent=2)
        self.add_content("</div>")
        self.add_content("<div class='row justify-content-center'>\n")
        self.add_content("<input type='text' placeholder='search' id='corner_search'>\n", indent=2)
        self.add_content(
            "<button type='submit' onclick='{}(undefined, label=\"{}\")' "
            "style='cursor: pointer'>Submit</button>\n".format(code, label), indent=2)
        self.add_content(
            "<button class='w3-button w3-teal w3-xlarge' "
            "onclick='side_bar_open()' style='cursor: pointer'>&#8801</button>\n", indent=2)
        self.add_content("</div>\n")
        self.add_content("<div class='row justify-content-center'>\n")
        if popular_options:
            for i in popular_options:
                if type(i) == dict and list(i.keys()) == ["all"]:
                    self.add_content("<button type='button' class='btn btn-info' "
                                     "onclick='{}(\"{}\", label=\"{}\")' "
                                     "style='margin-left:0.25em; margin-right:0.25em; "
                                     "margin-top: 1.0em; cursor: pointer'>all</button>\n".format(
                                         code, i["all"], label), indent=2)
                else:
                    self.add_content("<button type='button' class='btn btn-info' "
                                     "onclick='{}(\"{}\", label=\"{}\")' "
                                     "style='margin-left:0.25em; margin-right:0.25em; "
                                     "margin-top: 1.0em; cursor: pointer'>{}</button>\n".format(
                                         code, i, label, i), indent=2)
        self.add_content("</div>")
        self.add_content("<div class='container' style='margin-top:5em; margin-bottom:5em;"
                         "background-color:#FFFFFF; box-shadow: 0 0 5px grey;'>\n")
        self.add_content("<div class='row justify-content-center' id='corner_plot'>\n", indent=2)
        self.add_content("<canvas id='{}' width='600' height='600'></canvas>\n".format(ids), indent=4)
        if code == "combine":
            self.add_content("<img src='' id='mirror'/>", indent=4)
        self.add_content("</div>\n", indent=2)
        self.add_content("</div>\n")

    def make_modal_carousel(self, images=None):
        """Make a pop up window that appears on top of the home page showing
        images in a carousel.

        Parameters
        ----------
        images: list
            list of image locations that you would like included in the
            carousel
        """
        self.add_content("<div class='modal fade bs-example-modal-lg' tabindex='-1' "
                         "role='dialog' aria-labelledby='myLargeModalLabel' "
                         "aria-hidden='true' id='myModel' style='margin-top: 200px;'>\n")
        self.add_content("<div class='modal-dialog modal-lg' style='width:90%'>\n", indent=2)
        self.add_content("<div class='modal-content'>\n", indent=4)
        self.add_content("<div id='demo' class='carousel slide' data-ride='carousel'"
                         " data-interval='false'>\n", indent=6)
        self.add_content("<ul class='carousel-indicators'>\n", indent=8)
        for num, i in enumerate(images):
            if num == 0:
                self.add_content("<li data-target='#demo' data-slide-to-'%s' "
                                 "class='active'></li>\n" % (num), indent=10)
            self.add_content("<li data-target='#demo' data-slide-to-'%s'>"
                             "</li>\n" % (num), indent=10)
        self.add_content("<li data-target='#demo' data-slide-to='0' "
                         "class='active'></li>\n", indent=10)
        self.add_content("<li data-target='#demo' data-slide-to='1'></li>\n", indent=10)
        self.add_content("<li data-target='#demo' data-slide-to='2'></li>\n", indent=10)
        self.add_content("</ul>\n", indent=8)
        self.add_content("<div class='carousel-inner'>\n", indent=8)
        for num, i in enumerate(images):
            if num == 0:
                self.add_content("<div class='carousel-item active'>\n", indent=10)
            else:
                self.add_content("<div class='carousel-item'>\n", indent=10)
            self.add_content("<img src={} style='align-items:center;' "
                             "class='mx-auto d-block'>\n".format(i), indent=12)
            self.add_content("</div>\n", indent=10)

        self.add_content("</div>\n", indent=8)
        self.add_content("<a class='carousel-control-prev' href='#demo' "
                         "data-slide='prev'>\n", indent=8)
        self.add_content("<span class='carousel-control-prev-icon'></span>\n", indent=10)
        self.add_content("</a>\n", indent=8)
        self.add_content("<a class='carousel-control-next' href='#demo' "
                         "data-slide='next'>\n", indent=8)
        self.add_content("<span class='carousel-control-next-icon'></span>\n", indent=10)
        self.add_content("</a>\n", indent=8)
        self.add_content("</div>\n", indent=6)
        self.add_content("</div>\n", indent=4)
        self.add_content("</div>\n", indent=2)
        self.add_content("</div>\n")

    def make_cli_button(self, cli):
        """Make a button showing the command line used

        Parameters
        ----------
        cli: str
            the command line that you wish to display in the modal
        """
        self.make_div(0, _class="imgButton", _style=None)
        self.add_content("<button value='test' data-toggle='modal' "
                         "data-target='#myModal'>Command Line<"
                         "/button>", indent=2)
        self.end_div(0)
        self.add_content("<div class='modal' id='myModal'>", indent=0)
        self.make_div(2, _class='modal-dialog',
                      _style="max-width: 1000px; padding-top: 250px;")
        self.make_div(4, _class='modal-content', _style=None)
        self.make_div(6, _class='modal-body',
                      _style="font-size: 0.75rem;")
        self.add_content("%s" % (cli), indent=8)
        self.end_div(6)
        self.end_div(4)
        self.end_div(2)
        self.end_div(0)



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import pesummary
from pesummary.gw.file.formats.base_read import GWRead
from pesummary.core.file.formats.pesummary import PESummary as CorePESummary
from pesummary.utils.utils import logger, Array

import os
from glob import glob
import numpy as np


class PESummary(CorePESummary):
    """This class handles the existing posterior_samples.h5 file

    Parameters
    ----------
    parser: argparser
        The parser containing the command line arguments

    Attributes
    ----------
    existing_file: str
        the path to the existing posterior_samples.h5 file
    existing_approximants: list
        list of approximants that have been used in the previous analysis
    existing_labels: list
        list of labels that have been used in the previous analysis
    existing_samples: nd list
        nd list of samples stored for each approximant used in the previous
        analysis
    """
    def __init__(self, path_to_results_file, **kwargs):
        super(PESummary, self).__init__(path_to_results_file)
        load_kwargs = {
            "grab_data_from_dictionary": PESummary._grab_data_from_dictionary}
        self.load(self._grab_data_from_pesummary_file, **load_kwargs)
        self.samples_dict = None

    @classmethod
    def load_file(cls, path):
        if os.path.isdir(path):
            files = glob(path + "/*")
            if "home.html" in files:
                path = glob(path + "/samples/posterior_samples*")[0]
            else:
                raise Exception(
                    "Unable to find a file called 'posterior_samples' in "
                    "the directory %s" % (path + "/samples"))
        if not os.path.isfile(path):
            raise Exception("%s does not exist" % (path))
        return cls(path)

    @staticmethod
    def _grab_data_from_dictionary(dictionary):
        """
        """
        labels = list(dictionary["posterior_samples"].keys())
        existing_structure = {
            i: j for i in labels for j in
            dictionary["posterior_samples"]["%s" % (i)].keys()}
        labels = list(existing_structure.keys())

        parameter_list, sample_list, approx_list, inj_list = [], [], [], []
        ver_list, meta_data_list, weights_list = [], [], []
        for num, i in enumerate(labels):
            p = [j for j in dictionary["posterior_samples"]["%s" % (i)]["parameter_names"]]
            s = [j for j in dictionary["posterior_samples"]["%s" % (i)]["samples"]]
            if "injection_data" in dictionary.keys():
                inj = [j for j in dictionary["injection_data"]["%s" % (i)]["injection_values"]]
                for num, j in enumerate(inj):
                    if isinstance(j, (str, bytes)):
                        if j.decode("utf-8") == "NaN":
                            inj[num] = float("nan")
                    elif isinstance(j, (list, np.ndarray)):
                        inj[num] = inj[num][0]
                inj_list.append({i: j for i, j in zip(p, inj)})
            if isinstance(p[0], bytes):
                parameter_list.append([j.decode("utf-8") for j in p])
            else:
                parameter_list.append([j for j in p])
            sample_list.append(s)
            psd, cal, config = None, None, None
            if "config_file" in dictionary.keys():
                config, = GWRead.load_recusively("config_file", dictionary)
            if "psds" in dictionary.keys():
                psd, = GWRead.load_recusively("psds", dictionary)
            if "calibration_envelope" in dictionary.keys():
                cal, = GWRead.load_recusively("calibration_envelope", dictionary)
            if "approximant" in dictionary.keys():
                approx_list.append(dictionary["approximant"]["%s" % (i)])
            else:
                approx_list.append(None)
            if "meta_data" in dictionary.keys():
                data, = GWRead.load_recusively("meta_data", dictionary)
                meta_data_list.append(data["%s" % (i)])
            else:
                meta_data_list.append({"sampler": {}, "meta_data": {}})
            if "weights" in p or b"weights" in p:
                ind = p.index("weights") if "weights" in p else p.index(b"weights")
                weights_list.append(Array([j[ind] for j in s]))
            else:
                weights_list.append(None)

        if "version" in dictionary.keys():
            version, = GWRead.load_recusively("version", dictionary)
        else:
            version = {i: "No version information found" for i in labels
                       + ["pesummary"]}
        if "priors" in dictionary.keys():
            priors, = GWRead.load_recusively("priors", dictionary)
        else:
            priors = {}
        for i in list(version.keys()):
            if i != "pesummary" and isinstance(version[i][0], bytes):
                ver_list.append(version[i][0].decode("utf-8"))
            elif i != "pesummary":
                ver_list.append(version[i][0])
            elif isinstance(version["pesummary"], bytes):
                version["pesummary"] = version["pesummary"].decode("utf-8")
        setattr(PESummary, "labels", labels)
        setattr(PESummary, "config", config)
        setattr(PESummary, "psd", psd)
        setattr(PESummary, "calibration", cal)
        setattr(PESummary, "approximant", approx_list)
        setattr(PESummary, "version", version["pesummary"])
        setattr(PESummary, "priors", priors)
        return {
            "parameters": parameter_list,
            "samples": sample_list,
            "injection": inj_list,
            "version": ver_list,
            "kwargs": meta_data_list,
            "weights": {i: j for i, j in zip(labels, weights_list)}
        }

    @property
    def calibration_data_in_results_file(self):
        if self.calibration:
            keys = [list(self.calibration[i].keys()) for i in self.labels]
            total = [[self.calibration[key][ifo] for ifo in keys[num]] for
                     num, key in enumerate(self.labels)]
            return total, keys
        return None

    @property
    def detectors(self):
        det_list = []
        for i in self.parameters:
            detectors = []
            for param in i:
                if "_optimal_snr" in param and param != "network_optimal_snr":
                    detectors.append(param.split("_optimal_snr")[0])
            if detectors == []:
                detectors.append(None)
            det_list.append(detectors)
        return det_list

    def to_bilby(self):
        """Convert a PESummary metafile to a bilby results object
        """
        from bilby.gw.result import CompactBinaryCoalescenceResult
        from bilby.core.priors import PriorDict, Uniform
        from pandas import DataFrame

        objects = {}
        for num, i in enumerate(self.labels):
            priors = PriorDict()
            logger.warn(
                "No prior information is known so setting it to a default")
            priors.update({i: Uniform(-10, 10, 0) for i in self.parameters[num]})
            posterior_data_frame = DataFrame(
                self.samples[num], columns=self.parameters[num])
            meta_data = {
                "likelihood": {
                    "waveform_arguments": {
                        "waveform_approximant": self.approximant[num]},
                    "interferometers": self.detectors[num]}}
            bilby_object = CompactBinaryCoalescenceResult(
                search_parameter_keys=self.existing_parameters[num],
                posterior=posterior_data_frame, label="pesummary_%s" % (i),
                samples=self.existing_samples[num],
                meta_data=meta_data, priors=priors)
            objects[i] = bilby_object
        return objects

    def to_lalinference(self, outdir="./"):
        """Save a PESummary metafile as a lalinference hdf5 file
        """
        import numpy as np
        import h5py

        for num, i in enumerate(self.existing_labels):
            lalinference_samples = np.array(
                [tuple(i) for i in self.existing_samples[num]],
                dtype=[(i, '<f4') for i in self.existing_parameters[num]])

            try:
                f = h5py.File("%s/lalinference_file_%s.hdf5" % (outdir, i), "w")
            except Exception:
                raise Exception("Please make sure you have write permission in "
                                "%s" % (outdir))
            lalinference = f.create_group("lalinference")
            sampler = lalinference.create_group("lalinference_sampler")
            samples = sampler.create_dataset(
                "posterior_samples", data=lalinference_samples)
            f.close()



class InputError(Exception):
    """
    """
    def __init__(self, message):
        super(InputError, self).__init__(message)


class PlotError(Exception):
    """
    """
    def __init__(self, message):
        super(PlotError, self).__init__(message)



#! /usr/bin/env python

# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import os
from pesummary.gw.file.read import read as GWRead
from pesummary.core.file.read import read as Read
from pesummary.gw.file.meta_file import _GWMetaFile
from pesummary.core.file.meta_file import _MetaFile
from pesummary.gw.inputs import GWPostProcessing
from pesummary.gw.command_line import DictionaryAction
from pesummary.utils.utils import make_dir

import numpy as np

__doc__ = """This executable is used to combine multiple result files into a
single PESummary metafile"""


def command_line():
    """Generate an Argument Parser object to control the command line options
    """
    import argparse

    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("-w", "--webdir", dest="webdir",
                        help="make page and plots in DIR", metavar="DIR",
                        default="./")
    parser.add_argument("-s", "--samples", dest="samples", nargs='+',
                        help="Path to PESummary metafile",
                        default=None)
    parser.add_argument("--labels", dest="labels",
                        help="labels used to distinguish runs", nargs='+',
                        default=None)
    parser.add_argument("-c", "--config", dest="config",
                        help=("configuration file associcated with "
                              "each samples file."),
                        nargs='+', default=None)
    parser.add_argument("-i", "--inj_file", dest="inj_file",
                        help="path to injetcion file", nargs='+', default=None)
    parser.add_argument("--save_to_hdf5", action="store_true",
                        help="save the meta file in hdf5 format", default=False)

    gw_group = parser.add_argument_group(
        "Options specific for gravitational wave results files")
    gw_group.add_argument("-a", "--approximant", dest="approximant",
                          help=("waveform approximant used to generate "
                                "samples"), nargs='+', default=None)
    gw_group.add_argument("--psd", dest="psd", action=DictionaryAction,
                          help="psd files used", nargs='+', default=None)
    gw_group.add_argument("--calibration", dest="calibration",
                          help="files for the calibration envelope",
                          nargs="+", action=DictionaryAction, default=None)
    gw_group.add_argument("--trigfile", dest="inj_file",
                          help="xml file containing the trigger values",
                          nargs='+', default=None)
    gw_group.add_argument("--gw", action="store_true",
                          help="run with the gravitational wave pipeline",
                          default=False)
    return parser


def grab_information_from_file(samples, read_function, required_parameters,
                               config_file=None, injection_file=None):
    """Grab the information stored in result files

    Parameters
    ----------
    samples: list
        list of file paths
    read_function: function
        function to read in the result file. Should be either
        pesummary.gw.file.read.read or pesummary.core.file.read.read
    required_parameters: list
        list of attributes that you would like to return from the result file
    """
    full_list = {i: [] for i in required_parameters + ["injection_parameters"]}

    for idx, i in enumerate(samples):
        f = read_function(i)
        if config_file is not None and config_file[idx]:
            f.add_fixed_parameters_from_config_file(config_file[idx])
        if injection_file is not None and injection_file[idx]:
            f.add_injection_parameters_from_file(injection_file[idx])
        f.generate_all_posterior_samples()
        mydict = {j: getattr(f, j) for j in required_parameters}
        for j in required_parameters:
            full_list[j].append(mydict[j])
        if hasattr(f, "injection_parameters"):
            injection = f.injection_parameters
            if injection is not None:
                for i in mydict["parameters"]:
                    if i not in list(injection.keys()):
                        injection[i] = float("nan")
            else:
                injection = {i: j for i, j in zip(
                    mydict["parameters"],
                    [float("nan")] * len(mydict["parameters"]))
                }
        else:
            injection = {i: j for i, j in zip(
                mydict["parameters"],
                [float("nan")] * len(mydict["parameters"]))
            }
        full_list["injection_parameters"].append(injection)
    return full_list


def get_psd_data(psd, number=1):
    """Return the psd data passed from the command line

    Parameters
    ----------
    psd: list/dict
        paths to the psd files
    number: int, optional
        the number of result file passed
    """
    psd_data_list = []
    if isinstance(psd, dict):
        ifos = list(psd.keys())
        psd_labels = [ifos] * number
    else:
        psd_labels = [
            [GWPostProcessing._IFO_from_file_name(i) for i in psd]
        ] * number
        psd = {i: j for i, j in zip(psd_labels[0], psd)}
    for num, i in enumerate(psd_labels):
        psd_data = {}
        for j in i:
            if isinstance(psd[j], list):
                psd_data[j] = np.genfromtxt(psd[j][num], skip_footer=1).tolist()
            else:
                psd_data[j] = np.genfromtxt(psd[j], skip_footer=1).tolist()
        psd_data_list.append(psd_data)
    return psd_data_list


def get_calibration_data(calibration, number=1):
    """Return the calibration data passed from the command line

    Parameters
    ----------
    calibration: list/dict
        paths to the calibration envelopes
    number: int, optional
        the number of result file passed
    """
    calibration_data_list = []
    if isinstance(calibration, dict):
        ifos = list(calibration.keys())
        calibration_labels = [ifos] * number
    else:
        calibration_labels = [
            [GWPostProcessing._IFO_from_file_name(i) for i in calibration]
        ] * number
        calibration = {i: j for i, j in zip(calibration_labels[0], calibration)}
    for num, i in enumerate(calibration_labels):
        calibration_data = {}
        for j in i:
            if isinstance(calibration[j], list):
                calibration_data[j] = np.genfromtxt(calibration[j][num]).tolist()
            else:
                calibration_data[j] = np.genfromtxt(calibration[j]).tolist()
        calibration_data_list.append(calibration_data)
    return calibration_data_list


def gw_meta_file(opts):
    """Read in the files with the pesummary.gw.file.read.read function and
    return a pesummary.gw.file.meta_file._GWMetaFile object

    Parameters
    ----------
    opts: argparse.Namespace
        argparse namespace object that contains the passed arguments
    """
    read_function = GWRead
    required_parameters = [
        "parameters", "samples", "extra_kwargs", "input_version"]
    data = grab_information_from_file(
        opts.samples, read_function, required_parameters,
        config_file=opts.config, injection_file=opts.inj_file)
    data["labels"] = opts.labels

    if opts.psd is None:
        data["psd"] = [None for i in range(len(data["labels"]))]
    else:
        data["psd"] = get_psd_data(opts.psd, number=len(data["labels"]))
    if opts.calibration is None:
        data["calibration"] = [None for i in range(len(data["labels"]))]
    else:
        data["calibration"] = get_calibration_data(
            opts.calibration, number=len(data["labels"])
        )
    if opts.approximant is None:
        data["approximant"] = [None for i in range(len(data["labels"]))]
    else:
        data["approximant"] = opts.approximant
    data["config"] = opts.config

    meta_file = _GWMetaFile(data["parameters"], data["samples"],
                            data["labels"], data["config"],
                            data["injection_parameters"],
                            data["input_version"], data["extra_kwargs"],
                            calibration=data["calibration"], psd=data["psd"],
                            approximant=data["approximant"], webdir=opts.webdir)
    return meta_file


def core_meta_file(opts):
    """Read in the files with the pesummary.core.file.read.read function and
    return a pesummary.core.file.meta_file._MetaFile object

    Parameters
    ----------
    opts: argparse.Namespace
        argparse namespace object that contains the passed arguments
    """
    read_function = Read
    required_parameters = [
        "parameters", "samples", "extra_kwargs", "input_version"]
    data = grab_information_from_file(
        opts.samples, read_function, required_parameters,
        config_file=opts.config, injection_file=opts.inj_file)
    data["labels"] = opts.labels
    data["config"] = opts.config

    meta_file = _MetaFile(data["parameters"], data["samples"], data["labels"],
                          data["config"], data["injection_parameters"],
                          data["input_version"], data["extra_kwargs"],
                          webdir=opts.webdir)
    return meta_file


def main():
    """Top level interface for `summarycombine_metafiles`
    """
    parser = command_line()
    opts = parser.parse_args()

    if opts.webdir and not os.path.isdir(opts.webdir + "/samples"):
        make_dir(opts.webdir + "/samples")

    if opts.gw:
        meta_file = gw_meta_file(opts)
    else:
        meta_file = core_meta_file(opts)

    meta_file.generate_meta_file_data()
    if not opts.save_to_hdf5:
        meta_file.save_to_json()
    else:
        meta_file.save_to_hdf5()


if __name__ == "__main__":
    main()



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import os
import numpy as np
import json

import pesummary
from pesummary import __version__
from pesummary.core.inputs import PostProcessing
from pesummary.utils.utils import make_dir, logger
from pesummary import conf


def _recursively_save_dictionary_to_hdf5_file(f, dictionary, current_path=None):
    """Recursively save a dictionary to a hdf5 file

    Parameters
    ----------
    f: h5py._hl.files.File
        the open hdf5 file that you would like the data to be saved to
    dictionary: dict
        dictionary of data
    current_path: optional, str
        string to indicate the level to save the data in the hdf5 file
    """
    try:
        f.create_group("posterior_samples")
        if "config_file" in dictionary.keys():
            f.create_group("config_file")
        if "injection_data" in dictionary.keys():
            f.create_group("injection_data")
        if "version" in dictionary.keys():
            f.create_group("version")
        if "meta_data" in dictionary.keys():
            f.create_group("meta_data")
    except Exception:
        pass
    if current_path is None:
        current_path = []

    for k, v in dictionary.items():
        if isinstance(v, dict):
            try:
                f["/".join(current_path)].create_group(k)
            except Exception:
                pass
            path = current_path + [k]
            _recursively_save_dictionary_to_hdf5_file(f, v, path)
        elif isinstance(v, (list, pesummary.utils.utils.Array, np.ndarray)):
            import math

            if isinstance(v[0], str):
                f["/".join(current_path)].create_dataset(k, data=np.array(
                    v, dtype="S"
                ))
            elif isinstance(v[0], (list, pesummary.utils.utils.Array, np.ndarray)):
                f["/".join(current_path)].create_dataset(k, data=np.array(v))
            elif math.isnan(v[0]):
                f["/".join(current_path)].create_dataset(k, data=np.array(
                    ["NaN"] * len(v), dtype="S"
                ))
            elif isinstance(v[0], float):
                f["/".join(current_path)].create_dataset(k, data=np.array(v))
        elif isinstance(v, (str, bytes)):
            f["/".join(current_path)].create_dataset(k, data=np.array(
                [v], dtype="S"
            ))
        elif isinstance(v, float):
            f["/".join(current_path)].create_dataset(k, data=np.array(
                [v], dtype='<f4'))
        elif v == {}:
            f["/".join(current_path)].create_dataset(k, data=np.array("NaN"))


class PESummaryJsonEncoder(json.JSONEncoder):
    """Personalised JSON encoder for PESummary
    """
    def default(self, obj):
        """Return a json serializable object for 'obj'

        Parameters
        ----------
        obj: object
            object you wish to make json serializable
        """
        if isinstance(obj, np.ndarray):
            return obj.tolist()
        return json.JSONEncoder.default(self, obj)


class _MetaFile(object):
    """This is a base class to handle the functions to generate a meta file
    """
    def __init__(
        self, samples, labels, config, injection_data, file_versions,
        file_kwargs, webdir=None, result_files=None, hdf5=False, priors={},
        existing_version=None, existing_label=None, existing_samples=None,
        existing_injection=None, existing_metadata=None, existing_config=None,
        existing_priors={}, existing_metafile=None, outdir=None, existing=None
    ):
        self.data = {}
        self.webdir = webdir
        self.result_files = result_files
        self.samples = samples
        self.labels = labels
        self.config = config
        self.injection_data = injection_data
        self.file_versions = file_versions
        self.file_kwargs = file_kwargs
        self.hdf5 = hdf5
        self.priors = priors
        self.existing_version = existing_version
        self.existing_labels = existing_label
        self.existing_samples = existing_samples
        self.existing_injection = existing_injection
        self.existing_file_kwargs = existing_metadata
        self.existing_config = existing_config
        self.existing_priors = existing_priors
        self.existing_metafile = existing_metafile
        self.existing = existing
        self.outdir = outdir

        if self.existing_labels is None:
            self.existing_labels = [None]
        if self.existing is not None:
            self.add_existing_data()

    @property
    def outdir(self):
        return self._outdir

    @outdir.setter
    def outdir(self, outdir):
        if outdir is not None:
            self._outdir = os.path.abspath(outdir)
        elif self.webdir is not None:
            self._outdir = os.path.join(self.webdir, "samples")
        else:
            raise Exception("Please provide an output directory for the data")

    @property
    def file_name(self):
        base = "posterior_samples.{}"
        if self.hdf5:
            return base.format("h5")
        return base.format("json")

    @property
    def meta_file(self):
        return os.path.join(self.outdir, self.file_name)

    def make_dictionary(self):
        """Wrapper function for _make_dictionary
        """
        self._make_dictionary()

    def _make_dictionary(self):
        """Generate a single dictionary which stores all information
        """
        dictionary = {
            "posterior_samples": {},
            "injection_data": {},
            "version": {},
            "meta_data": {},
            "priors": {},
            "config_file": {}
        }

        dictionary["priors"] = self.priors
        for num, label in enumerate(self.labels):
            parameters = self.samples[label].keys()
            samples = np.array([self.samples[label][i] for i in parameters]).T
            dictionary["posterior_samples"][label] = {
                "parameter_names": list(parameters),
                "samples": samples.tolist()
            }

            dictionary["injection_data"][label] = {
                "injection_values": [
                    self.injection_data[label][i] for i in parameters
                ]
            }

            dictionary["version"][label] = [self.file_versions[label]]
            dictionary["version"]["pesummary"] = [__version__]
            dictionary["meta_data"][label] = self.file_kwargs[label]

            if self.config != {} and self.config[num] is not None and not isinstance(self.config[num], dict):
                config = self._grab_config_data_from_data_file(self.config[num])
                dictionary["config_file"][label] = config
            elif self.config[num] is not None:
                dictionary["config_file"][label] = self.config[num]
        self.data = dictionary

    @staticmethod
    def _grab_config_data_from_data_file(file):
        """Return the config data as a dictionary

        Parameters
        ----------
        file: str
            path to the configuration file
        """
        import configparser

        data = {}
        config = configparser.ConfigParser()
        try:
            config.read(file)
            sections = config.sections()
        except Exception as e:
            sections = None
            logger.info(
                "Unable to open %s because %s. The data will not be stored in "
                "the meta file" % (file, e)
            )
        if sections:
            for i in sections:
                data[i] = {}
                for key in config["%s" % (i)]:
                    data[i][key] = config["%s" % (i)]["%s" % (key)]
        return data

    @staticmethod
    def write_to_dat(file_name, samples, header=None):
        """Write samples to a .dat file

        Parameters
        ----------
        file_name: str
            the name of the file that you wish to write the samples to
        samples: np.ndarray
            1d/2d array of samples to write to file
        header: list, optional
            List of strings to write at the beginning of the file
        """
        np.savetxt(
            file_name, samples, delimiter=conf.delimiter,
            header=conf.delimiter.join(header), comments=""
        )

    def write_marginalized_posterior_to_dat(self):
        """Write the marginalized posterior for each parameter to a .dat file
        """
        for label in self.labels:
            if not os.path.isdir(os.path.join(self.outdir, label)):
                make_dir(os.path.join(self.outdir, label))
            for param, samples in self.samples[label].items():
                self.write_to_dat(
                    os.path.join(
                        self.outdir, label, "{}_{}.dat".format(label, param)
                    ), samples, header=[param]
                )

    def save_to_json(self):
        """Save the metafile as a json file
        """
        with open(self.meta_file, "w") as f:
            json.dump(self.data, f, indent=4, sort_keys=True, cls=PESummaryJsonEncoder)

    def save_to_hdf5(self):
        """Save the metafile as a hdf5 file
        """
        import h5py

        with h5py.File(self.meta_file, "w") as f:
            _recursively_save_dictionary_to_hdf5_file(f, self.data)

    def save_to_dat(self):
        """Save the samples to a .dat file
        """
        for label in self.labels:
            parameters = self.samples[label].keys()
            samples = np.array([self.samples[label][i] for i in parameters])
            self.write_to_dat(
                os.path.join(self.outdir, "{}_pesummary.dat".format(label)),
                samples.T, header=parameters
            )

    def add_existing_data(self):
        """
        """
        from pesummary.utils.utils import _add_existing_data

        self = _add_existing_data(self)


class MetaFile(PostProcessing):
    """This class handles the creation of a metafile storing all information
    from the analysis
    """
    def __init__(self, inputs):
        from pesummary.utils.utils import logger

        super(MetaFile, self).__init__(inputs)
        logger.info("Starting to generate the meta file")
        meta_file = _MetaFile(
            self.samples, self.labels, self.config,
            self.injection_data, self.file_version, self.file_kwargs,
            hdf5=self.hdf5, webdir=self.webdir, result_files=self.result_files,
            existing_version=self.existing_file_version, existing_label=self.existing_labels,
            existing_samples=self.existing_samples,
            existing_injection=self.existing_injection_data,
            existing_metadata=self.existing_file_kwargs,
            existing_config=self.existing_config, existing=self.existing,
            existing_priors=self.existing_priors,
            existing_metafile=self.existing_metafile
        )
        meta_file.make_dictionary()
        if not self.hdf5:
            meta_file.save_to_json()
        else:
            meta_file.save_to_hdf5()
        meta_file.save_to_dat()
        meta_file.write_marginalized_posterior_to_dat()
        logger.info(
            "Finishing generating the meta file. The meta file can be viewed "
            "here: {}".format(meta_file.meta_file)
        )



# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

from pesummary.utils.utils import logger
from pesummary.core.plots.kde import kdeplot
from pesummary import conf

import matplotlib
matplotlib.use("Agg")
import matplotlib.pyplot as plt
import corner
import copy

import numpy as np
from scipy import signal


def _autocorrelation_plot(param, samples):
    """Generate the autocorrelation function for a set of samples for a given
    parameter for a given approximant.

    Parameters
    ----------
    param: str
        name of the parameter that you wish to plot
    samples: list
        list of samples for param
    """
    logger.debug("Generating the autocorrelation function for %s" % (param))
    samples = samples[int(len(samples) / 2):]
    x = samples - np.mean(samples)
    y = np.conj(x[::-1])
    acf = np.fft.ifftshift(signal.fftconvolve(y, x, mode='full'))
    N = np.array(samples).shape[0]
    acf = acf[0:N]
    fig = plt.figure()
    plt.plot(acf / acf[0], linestyle=' ', marker='o', markersize=0.5,
             color=conf.color)
    plt.ticklabel_format(axis="x", style="plain")
    plt.xlabel("lag", fontsize=16)
    plt.ylabel("ACF", fontsize=16)
    plt.tight_layout()
    plt.grid(b=True)
    return fig


def _sample_evolution_plot(param, samples, latex_label, inj_value=None):
    """Generate a scatter plot showing the evolution of the samples for a
    given parameter for a given approximant.

    Parameters
    ----------
    param: str
        name of the parameter that you wish to plot
    samples: list
        list of samples for param
    latex_label: str
        latex label for param
    inj_value: float
        value that was injected
    """
    logger.debug("Generating the sample scatter plot for %s" % (param))
    fig = plt.figure()
    n_samples = len(samples)
    plt.plot(range(n_samples), samples, linestyle=' ', marker='o',
             markersize=0.5, color=conf.color)
    plt.ticklabel_format(axis="x", style="plain")
    plt.xlabel("samples", fontsize=16)
    plt.ylabel(latex_label, fontsize=16)
    plt.tight_layout()
    plt.grid(b=True)
    return fig


def _1d_cdf_plot(param, samples, latex_label):
    """Generate the cumulative distribution function for a given parameter for
    a given approximant.

    Parameters
    ----------
    param: str
        name of the parameter that you wish to plot
    samples: list
        list of samples for param
    latex_label: str
        latex label for param
    """
    logger.debug("Generating the 1d CDF for %s" % (param))
    fig = plt.figure()
    sorted_samples = copy.deepcopy(samples)
    sorted_samples.sort()
    plt.xlabel(latex_label, fontsize=16)
    plt.ylabel("Cumulative Density Function", fontsize=16)
    upper_percentile = np.percentile(samples, 90)
    lower_percentile = np.percentile(samples, 10)
    median = np.median(samples)
    upper = np.round(upper_percentile - median, 2)
    lower = np.round(median - lower_percentile, 2)
    median = np.round(median, 2)
    plt.title(r"$%s^{+%s}_{-%s}$" % (median, upper, lower), fontsize=18)
    plt.plot(sorted_samples, np.linspace(0, 1, len(sorted_samples)),
             color=conf.color)
    plt.grid(b=True)
    plt.ylim([0, 1.05])
    plt.tight_layout()
    return fig


def _1d_cdf_comparison_plot(param, samples, colors, latex_label, labels):
    """Generate a plot to compare the cdfs for a given parameter for different
    approximants.

    Parameters
    ----------
    param: str
        name of the parameter that you wish to plot
    approximants: list
        list of approximant names that you would like to compare
    samples: 2d list
        list of samples for param for each approximant
    colors: list
        list of colors to be used to differentiate the different approximants
    latex_label: str
        latex label for param
    approximant_labels: list, optional
        label to prepend the approximant in the legend
    """
    logger.debug("Generating the 1d comparison CDF for %s" % (param))
    fig = plt.figure(figsize=(8, 6))
    for num, i in enumerate(samples):
        sorted_samples = copy.deepcopy(samples[num])
        sorted_samples = sorted(sorted_samples)
        plt.plot(sorted_samples, np.linspace(0, 1, len(sorted_samples)),
                 color=colors[num], label=labels[num])
    plt.xlabel(latex_label, fontsize=16)
    plt.ylabel("Cumulative Density Function", fontsize=16)
    plt.grid(b=True)
    plt.ylim([0, 1.05])
    plt.legend(bbox_to_anchor=(0., 1.02, 1., .102), loc=3,
               ncol=2, mode="expand", borderaxespad=0.)
    plt.tight_layout()
    return fig


def _1d_histogram_plot(param, samples, latex_label, inj_value=None, kde=False,
                       prior=None, weights=None):
    """Generate the 1d histogram plot for a given parameter for a given
    approximant.

    Parameters
    ----------
    param: str
        name of the parameter that you wish to plot
    samples: list
        list of samples for param
    latex_label: str
        latex label for param
    inj_value: float
        value that was injected
    kde: Bool
        if true, a kde is plotted instead of a histogram
    prior: list
        list of prior samples for param
    weights: list
        list of weights for each sample
    """
    logger.debug("Generating the 1d histogram plot for %s" % (param))
    fig = plt.figure()
    if np.ptp(samples) == 0:
        plt.axvline(samples[0], color=conf.color)
        xlims = plt.gca().get_xlim()
    elif not kde:
        plt.hist(samples, histtype="step", bins=50, color=conf.color,
                 density=True, linewidth=1.75, weights=weights)
        xlims = plt.gca().get_xlim()
        if prior is not None:
            plt.hist(prior, color=conf.prior_color, alpha=0.2, edgecolor="w",
                     density=True, linewidth=1.75, histtype="bar", bins=50)
    else:
        x = kdeplot(
            samples, color=conf.color, shade=True, alpha_shade=0.1,
            clip=[samples.minimum, samples.maximum], linewidth=1.0,
            weights=weights
        )
        xlims = plt.gca().get_xlim()
        if prior is not None:
            kdeplot(prior, color=conf.prior_color, shade=True, alpha_shade=0.1,
                    clip=[prior.minimum, prior.maximum], linewidth=1.0)
    plt.xlabel(latex_label, fontsize=16)
    plt.ylabel("Probability Density", fontsize=16)
    percentile = samples.confidence_interval([10, 90])
    if inj_value is not None:
        plt.axvline(inj_value, color=conf.injection_color, linestyle='-',
                    linewidth=2.5)
    plt.axvline(percentile[0], color=conf.color, linestyle='--', linewidth=1.75)
    plt.axvline(percentile[1], color=conf.color, linestyle='--', linewidth=1.75)
    median = samples.average("median")
    upper = np.round(percentile[1] - median, 2)
    lower = np.round(median - percentile[0], 2)
    median = np.round(median, 2)
    plt.title(r"$%s^{+%s}_{-%s}$" % (median, upper, lower), fontsize=18)
    plt.grid(b=True)
    plt.xlim(xlims)
    plt.tight_layout()
    return fig


def _1d_comparison_histogram_plot(param, samples, colors,
                                  latex_label, labels, kde=False):
    """Generate the a plot to compare the 1d_histogram plots for a given
    parameter for different approximants.

    Parameters
    ----------
    param: str
        name of the parameter that you wish to plot
    approximants: list
        list of approximant names that you would like to compare
    samples: 2d list
        list of samples for param for each approximant
    colors: list
        list of colors to be used to differentiate the different approximants
    latex_label: str
        latex label for param
    approximant_labels: list, optional
        label to prepend the approximant in the legend
    kde: Bool
        if true, a kde is plotted instead of a histogram
    """
    logger.debug("Generating the 1d comparison histogram plot for %s" % (param))
    fig = plt.figure(figsize=(8, 6))
    for num, i in enumerate(samples):
        if np.ptp(i) == 0:
            plt.axvline(i[0], color=colors[num], label=labels[num])
        elif not kde:
            plt.hist(i, histtype="step", bins=50, color=colors[num],
                     label=labels[num], linewidth=2.5, density=True)
        else:
            kdeplot(i, color=colors[num], shade=True, alpha_shade=0.05,
                    clip=[np.min(i), np.max(i)], linewidth=1.5,
                    label=labels[num])
        plt.axvline(x=np.percentile(i, 90), color=colors[num], linestyle='--',
                    linewidth=2.5)
        plt.axvline(x=np.percentile(i, 10), color=colors[num], linestyle='--',
                    linewidth=2.5)
    plt.xlabel(latex_label, fontsize=16)
    plt.ylabel("Probability Density", fontsize=16)
    plt.legend(bbox_to_anchor=(0., 1.02, 1., .102), loc=3,
               ncol=2, mode="expand", borderaxespad=0.)
    plt.grid(b=True)
    plt.tight_layout()
    return fig


def _comparison_box_plot(param, samples, colors, latex_label, labels):
    """Generate a box plot to compare 1d_histograms for a given parameter

    Parameters
    ----------
    param: str
        name of the parameter that you wish to plot
    approximants: list
        list of approximant names that you would like to compare
    samples: 2d list
        list of samples for param for each approximant
    colors: list
        list of colors to be used to differentiate the different approximants
    latex_label: str
        latex label for param
    approximant_labels: list, optional
        label to prepend the approximant in the legend
    """
    logger.debug("Generating the 1d comparison boxplot plot for %s" % (param))
    fig = plt.figure()
    maximum = np.max([np.max(i) for i in samples])
    minimum = np.min([np.min(i) for i in samples])
    middle = (maximum + minimum) * 0.5
    plt.boxplot(samples, widths=0.2, vert=False, whis=np.inf, labels=labels)
    for num, i in enumerate(labels):
        plt.annotate(i, xy=(middle, 1), xytext=(middle, num + 1. + 0.2),
                     ha="center")
    plt.yticks([])
    plt.xlabel(latex_label, fontsize=16)
    plt.tight_layout()
    plt.grid(b=True)
    return fig


def _make_corner_plot(samples, latex_labels, **kwargs):
    """Generate the corner plots for a given approximant

    Parameters
    ----------
    opts: argparse
        argument parser object to hold all information from the command line
    samples: nd list
        nd list of samples for each parameter for a given approximant
    params: list
        list of parameters associated with each element in samples
    approximant: str
        name of approximant that was used to generate the samples
    latex_labels: dict
        dictionary of latex labels for each parameter
    """
    logger.debug("Generating the corner plot")
    # set the default kwargs
    default_kwargs = dict(
        bins=50, smooth=0.9, label_kwargs=dict(fontsize=16),
        title_kwargs=dict(fontsize=16), color='#0072C1',
        truth_color='tab:orange', quantiles=[0.16, 0.84],
        levels=(1 - np.exp(-0.5), 1 - np.exp(-2), 1 - np.exp(-9 / 2.)),
        plot_density=False, plot_datapoints=True, fill_contours=True,
        max_n_ticks=3)
    parameters = list(samples.keys())
    xs = np.zeros([len(parameters), len(samples[parameters[0]])])
    for num, i in enumerate(parameters):
        xs[num] = samples[i]
    default_kwargs['range'] = [1.0] * len(parameters)
    default_kwargs["labels"] = [latex_labels[i] for i in parameters]
    figure = corner.corner(xs.T, **default_kwargs)
    # grab the axes of the subplots
    axes = figure.get_axes()
    extent = axes[0].get_window_extent().transformed(figure.dpi_scale_trans.inverted())
    width, height = extent.width, extent.height
    width *= figure.dpi
    height *= figure.dpi
    return figure, parameters



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
#                     Seaborn authors
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from seaborn.distributions import _bivariate_kdeplot, _scipy_univariate_kde
from seaborn.distributions import _statsmodels_univariate_kde
import warnings

try:
    import statsmodels.nonparametric.api as smnp
    _has_statsmodels = True
except ImportError:
    _has_statsmodels = False


def _univariate_kdeplot(data, shade, vertical, kernel, bw, gridsize, cut,
                        clip, legend, ax, cumulative=False, **kwargs):
    """Plot a univariate kernel density estimate on one of the axes."""

    # Sort out the clipping
    if clip is None:
        clip = (-np.inf, np.inf)

    # Calculate the KDE
    if _has_statsmodels:
        # Prefer using statsmodels for kernel flexibility
        x, y = _statsmodels_univariate_kde(data, kernel, bw,
                                           gridsize, cut, clip,
                                           cumulative=cumulative)
    else:
        # Fall back to scipy if missing statsmodels
        if kernel != "gau":
            kernel = "gau"
            msg = "Kernel other than `gau` requires statsmodels."
            warnings.warn(msg, UserWarning)
        if cumulative:
            raise ImportError("Cumulative distributions are currently "
                              "only implemented in statsmodels. "
                              "Please install statsmodels.")
        x, y = _scipy_univariate_kde(data, bw, gridsize, cut, clip)

    # Make sure the density is nonnegative
    y = np.amax(np.c_[np.zeros_like(y), y], axis=1)

    # Flip the data if the plot should be on the y axis
    if vertical:
        x, y = y, x

    # Check if a label was specified in the call
    label = kwargs.pop("label", None)
    alpha_shade = kwargs.pop("alpha_shade", 0.25)

    # Otherwise check if the data object has a name
    if label is None and hasattr(data, "name"):
        label = data.name

    # Decide if we're going to add a legend
    legend = label is not None and legend
    label = "_nolegend_" if label is None else label

    # Use the active color cycle to find the plot color
    facecolor = kwargs.pop("facecolor", None)
    line, = ax.plot(x, y, **kwargs)
    color = line.get_color()
    line.remove()
    kwargs.pop("color", None)
    facecolor = color if facecolor is None else facecolor

    # Draw the KDE plot and, optionally, shade
    ax.plot(x, y, color=color, label=label, **kwargs)
    shade_kws = dict(
        facecolor=facecolor,
        alpha=alpha_shade,
        clip_on=kwargs.get("clip_on", True),
        zorder=kwargs.get("zorder", 1),)
    if shade:
        if vertical:
            ax.fill_betweenx(y, 0, x, **shade_kws)
        else:
            ax.fill_between(x, 0, y, **shade_kws)

    # Set the density axis minimum to 0
    if vertical:
        ax.set_xlim(0, auto=None)
    else:
        ax.set_ylim(0, auto=None)

    # Draw the legend here
    handles, labels = ax.get_legend_handles_labels()
    if legend and handles:
        ax.legend(loc="best")

    return ax


def kdeplot(data, data2=None, shade=False, vertical=False, kernel="gau",
            bw="scott", gridsize=100, cut=3, clip=None, legend=True,
            cumulative=False, shade_lowest=True, cbar=False, cbar_ax=None,
            cbar_kws=None, ax=None, **kwargs):
    """Fit and plot a univariate or bivariate kernel density estimate.
    Parameters
    ----------
    data : 1d array-like
        Input data.
    data2: 1d array-like, optional
        Second input data. If present, a bivariate KDE will be estimated.
    shade : bool, optional
        If True, shade in the area under the KDE curve (or draw with filled
        contours when data is bivariate).
    vertical : bool, optional
        If True, density is on x-axis.
    kernel : {'gau' | 'cos' | 'biw' | 'epa' | 'tri' | 'triw' }, optional
        Code for shape of kernel to fit with. Bivariate KDE can only use
        gaussian kernel.
    bw : {'scott' | 'silverman' | scalar | pair of scalars }, optional
        Name of reference method to determine kernel size, scalar factor,
        or scalar for each dimension of the bivariate plot. Note that the
        underlying computational libraries have different interperetations
        for this parameter: ``statsmodels`` uses it directly, but ``scipy``
        treats it as a scaling factor for the standard deviation of the
        data.
    gridsize : int, optional
        Number of discrete points in the evaluation grid.
    cut : scalar, optional
        Draw the estimate to cut * bw from the extreme data points.
    clip : pair of scalars, or pair of pair of scalars, optional
        Lower and upper bounds for datapoints used to fit KDE. Can provide
        a pair of (low, high) bounds for bivariate plots.
    legend : bool, optional
        If True, add a legend or label the axes when possible.
    cumulative : bool, optional
        If True, draw the cumulative distribution estimated by the kde.
    shade_lowest : bool, optional
        If True, shade the lowest contour of a bivariate KDE plot. Not
        relevant when drawing a univariate plot or when ``shade=False``.
        Setting this to ``False`` can be useful when you want multiple
        densities on the same Axes.
    cbar : bool, optional
        If True and drawing a bivariate KDE plot, add a colorbar.
    cbar_ax : matplotlib axes, optional
        Existing axes to draw the colorbar onto, otherwise space is taken
        from the main axes.
    cbar_kws : dict, optional
        Keyword arguments for ``fig.colorbar()``.
    ax : matplotlib axes, optional
        Axes to plot on, otherwise uses current axes.
    kwargs : key, value pairings
        Other keyword arguments are passed to ``plt.plot()`` or
        ``plt.contour{f}`` depending on whether a univariate or bivariate
        plot is being drawn.
    Returns
    -------
    ax : matplotlib Axes
        Axes with plot.
    See Also
    --------
    distplot: Flexibly plot a univariate distribution of observations.
    jointplot: Plot a joint dataset with bivariate and marginal distributions.
    Examples
    --------
    Plot a basic univariate density:
    .. plot::
        :context: close-figs
        >>> import numpy as np; np.random.seed(10)
        >>> import seaborn as sns; sns.set(color_codes=True)
        >>> mean, cov = [0, 2], [(1, .5), (.5, 1)]
        >>> x, y = np.random.multivariate_normal(mean, cov, size=50).T
        >>> ax = sns.kdeplot(x)
    Shade under the density curve and use a different color:
    .. plot::
        :context: close-figs
        >>> ax = sns.kdeplot(x, shade=True, color="r")
    Plot a bivariate density:
    .. plot::
        :context: close-figs
        >>> ax = sns.kdeplot(x, y)
    Use filled contours:
    .. plot::
        :context: close-figs
        >>> ax = sns.kdeplot(x, y, shade=True)
    Use more contour levels and a different color palette:
    .. plot::
        :context: close-figs
        >>> ax = sns.kdeplot(x, y, n_levels=30, cmap="Purples_d")
    Use a narrower bandwith:
    .. plot::
        :context: close-figs
        >>> ax = sns.kdeplot(x, bw=.15)
    Plot the density on the vertical axis:
    .. plot::
        :context: close-figs
        >>> ax = sns.kdeplot(y, vertical=True)
    Limit the density curve within the range of the data:
    .. plot::
        :context: close-figs
        >>> ax = sns.kdeplot(x, cut=0)
    Add a colorbar for the contours:
    .. plot::
        :context: close-figs
        >>> ax = sns.kdeplot(x, y, cbar=True)
    Plot two shaded bivariate densities:
    .. plot::
        :context: close-figs
        >>> iris = sns.load_dataset("iris")
        >>> setosa = iris.loc[iris.species == "setosa"]
        >>> virginica = iris.loc[iris.species == "virginica"]
        >>> ax = sns.kdeplot(setosa.sepal_width, setosa.sepal_length,
        ...                  cmap="Reds", shade=True, shade_lowest=False)
        >>> ax = sns.kdeplot(virginica.sepal_width, virginica.sepal_length,
        ...                  cmap="Blues", shade=True, shade_lowest=False)
    """
    if ax is None:
        ax = plt.gca()

    if isinstance(data, list):
        data = np.asarray(data)

    if len(data) == 0:
        return ax

    data = data.astype(np.float64)
    if data2 is not None:
        if isinstance(data2, list):
            data2 = np.asarray(data2)
        data2 = data2.astype(np.float64)

    warn = False
    bivariate = False
    if isinstance(data, np.ndarray) and np.ndim(data) > 1:
        warn = True
        bivariate = True
        x, y = data.T
    elif isinstance(data, pd.DataFrame) and np.ndim(data) > 1:
        warn = True
        bivariate = True
        x = data.iloc[:, 0].values
        y = data.iloc[:, 1].values
    elif data2 is not None:
        bivariate = True
        x = data
        y = data2

    if warn:
        warn_msg = ("Passing a 2D dataset for a bivariate plot is deprecated "
                    "in favor of kdeplot(x, y), and it will cause an error in "
                    "future versions. Please update your code.")
        warnings.warn(warn_msg, UserWarning)

    if bivariate and cumulative:
        raise TypeError("Cumulative distribution plots are not"
                        "supported for bivariate distributions.")
    if bivariate:
        ax = _bivariate_kdeplot(x, y, shade, shade_lowest,
                                kernel, bw, gridsize, cut, clip, legend,
                                cbar, cbar_ax, cbar_kws, ax, **kwargs)
    else:
        ax = _univariate_kdeplot(data, shade, vertical, kernel, bw,
                                 gridsize, cut, clip, legend, ax,
                                 cumulative=cumulative, **kwargs)

    return ax



#! /usr/bin/env python

# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import pesummary
from pesummary.gw.file.read import read as GWRead
import argparse


__doc__ = """This executable returns a cleaned data file"""


def command_line():
    """Generate an Argument Parser object to control the command line options
    """
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("-w", "--webdir", dest="webdir",
                        help="make page and plots in DIR", metavar="DIR",
                        default=None)
    parser.add_argument("-s", "--samples", dest="samples",
                        help="Posterior samples hdf5 file", nargs='+',
                        default=None)
    parser.add_argument("--labels", dest="labels",
                        help="labels used to distinguish runs", nargs='+',
                        default=None)
    parser.add_argument("--file_format", dest="file_format",
                        help="Save the cleaned data in this format",
                        choices=["dat", "lalinference", "bilby"],
                        default="dat")
    return parser


def clean_data_file(path):
    """Clean the data file and return a PESummary result file object

    Parameters
    ----------
    path: str
        path to the result file
    """
    f = GWRead(path)
    f.generate_all_posterior_samples()
    return f


def save(pesummary_object, file_format, webdir=None, label=None):
    """Save the pesummary_object to a given format

    Parameters
    ----------
    pesummary_object: pesummary.gw.file.formats
        pesummary results file object
    file_format: str
        the file format that you wish to save the file as
    webdir: str
        directory to save the cleaned data file
    """
    if file_format == "dat":
        pesummary_object.to_dat(outdir=webdir, label=label)
    elif file_format == "lalinference":
        pesummary_object.to_lalinference(outdir=webdir, label=label)
    elif file_format == "bilby":
        pesummary_object.to_bilby()


def main():
    """Top level interface for `summaryclean`
    """
    parser = command_line()
    opts = parser.parse_args()
    if opts.labels:
        if len(opts.labels) != len(opts.samples):
            raise Exception("Please provide labels for all result files")
    for num, i in enumerate(opts.samples):
        f = clean_data_file(i)
        label = None
        if opts.labels:
            label = opts.labels[num]
        save(f, opts.file_format, webdir=opts.webdir, label=label)



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import argparse
import copy


class DictionaryAction(argparse.Action):
    """Class to extend the argparse.Action to handle dictionaries as input
    """
    def __init__(self, option_strings, dest, nargs=None, const=None,
                 default=None, type=None, choices=None, required=False,
                 help=None, metavar=None):
        super(DictionaryAction, self).__init__(
            option_strings=option_strings, dest=dest, nargs=nargs,
            const=const, default=default, type=str, choices=choices,
            required=required, help=help, metavar=metavar)

    def __call__(self, parser, namespace, values, option_string=None):
        bool = [True if ':' in value else False for value in values]
        if all(i is True for i in bool):
            setattr(namespace, self.dest, {})
        elif all(i is False for i in bool):
            setattr(namespace, self.dest, [])
        else:
            raise ValueError("Did not understand input")

        items = getattr(namespace, self.dest)
        items = copy.copy(items)
        for value in values:
            value = value.split(':')
            if len(value) > 2:
                value = [":".join(value[:-1]), value[-1]]
            if len(value) == 2:
                if value[0] in items.keys():
                    if not isinstance(items[value[0]], list):
                        items[value[0]] = [items[value[0]]]
                    items[value[0]].append(value[1])
                else:
                    items[value[0]] = value[1]
            elif len(value) == 1:
                items.append(value[0])
            else:
                raise ValueError("Did not understand input")
        setattr(namespace, self.dest, items)


def insert_gwspecific_option_group(parser):
    """Add gravitational wave related options to the optparser object

    Parameters
    ----------
    parser: object
        OptionParser instance.
    """
    gw_group = parser.add_argument_group(
        "Options specific for gravitational wave results files")

    gw_group.add_argument("-a", "--approximant", dest="approximant",
                          help=("waveform approximant used to generate "
                                "samples"), nargs='+', default=None)
    gw_group.add_argument("--sensitivity", action="store_true",
                          help="generate sky sensitivities for HL, HLV",
                          default=False)
    gw_group.add_argument("--gracedb", dest="gracedb",
                          help="gracedb of the event", default=None)
    gw_group.add_argument("--psd", dest="psd", action=DictionaryAction,
                          help="psd files used", nargs='+', default={})
    gw_group.add_argument("--calibration", dest="calibration",
                          help="files for the calibration envelope",
                          nargs="+", action=DictionaryAction, default={})
    gw_group.add_argument("--trigfile", dest="inj_file",
                          help="xml file containing the trigger values",
                          nargs='+', default=None)
    gw_group.add_argument("--gwdata", dest="gwdata",
                          help="channels and paths to strain cache files",
                          action=DictionaryAction, metavar="CHANNEL:CACHEFILE",
                          nargs="+", default=None)
    gw_group.add_argument("--multi_threading_for_skymap", action="store_true",
                          help=("use multi-threading to speed up generation of "
                                "ligo.skymap"), default=False)
    gw_group.add_argument("--nsamples_for_skymap", dest="nsamples_for_skymap",
                          help=("The number of samples used to generate the "
                                "ligo.skymap. These samples will be randomly "
                                "drawn from the posterior distributions"),
                          default=None)
    gw_group.add_argument("--no_ligo_skymap", action="store_true",
                          help="do not generate a skymap with ligo.skymap",
                          default=False)
    gw_group.add_argument("--gw", action="store_true",
                          help="run with the gravitational wave pipeline",
                          default=False)
    return gw_group



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

from pesummary.core.file.formats.base_read import Read
from pesummary.gw.file.formats.lalinference import LALInference
from pesummary.gw.file.formats.bilby import Bilby
from pesummary.gw.file.formats.default import Default
from pesummary.gw.file.formats.pesummary import PESummary
from pesummary.core.file.read import is_bilby_hdf5_file, is_bilby_json_file
from pesummary.core.file.read import is_pesummary_hdf5_file
from pesummary.core.file.read import is_pesummary_json_file
from pesummary.utils.utils import logger


def read(path):
    """Read in a results file.

    Parameters
    ----------
    path: str
        path to the results file
    """
    extension = Read.extension_from_path(path)

    if extension in ["hdf5", "h5", "hdf"]:
        if is_lalinference_file(path):
            return LALInference.load_file(path)
        elif is_bilby_hdf5_file(path):
            try:
                return Bilby.load_file(path)
            except ImportError:
                logger.warn(
                    "Failed to import `bilby`. Using default load")
                return Default.laad_file(path)
        elif is_pesummary_hdf5_file(path):
            try:
                return PESummary.load_file(path)
            except Exception:
                return Default.load_file(path)
        else:
            return Default.load_file(path)
    elif extension == "json":
        if is_bilby_json_file(path):
            try:
                return Bilby.load_file(path)
            except ImportError:
                logger.warn(
                    "Failed to import `bilby`. Using default load")
                return Default.load_file(path)
        elif is_pesummary_json_file(path):
            try:
                return PESummary.load_file(path)
            except Exception:
                return Default.load_file(path)
        else:
            return Default.load_file(path)
    else:
        return Default.load_file(path)


def is_lalinference_file(path):
    """Determine if the results file is a LALInference results file

    Parameters
    ----------
    path: str
        path to the results file
    """
    import h5py
    f = h5py.File(path, 'r')
    keys = list(f.keys())
    f.close()
    if "lalinference" in keys:
        return True
    return False



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

from pesummary.gw.plots.cmap import cylon

cylon()



import numpy as np
from scipy.stats import gaussian_kde as kde


default_bounds = {"luminosity_distance": {"low": 0.},
                  "geocent_time": {"low": 0.},
                  "dec": {"low": -np.pi / 2, "high": np.pi / 2},
                  "ra": {"low": 0., "high": 2 * np.pi},
                  "a_1": {"low": 0., "high": 1.},
                  "a_2": {"low": 0., "high": 1.},
                  "phi_jl": {"low": 0., "high": 2 * np.pi},
                  "phase": {"low": 0., "high": 2 * np.pi},
                  "psi": {"low": 0., "high": 2 * np.pi},
                  "iota": {"low": 0., "high": np.pi},
                  "tilt_1": {"low": 0., "high": np.pi},
                  "tilt_2": {"low": 0., "high": np.pi},
                  "phi_12": {"low": 0., "high": 2 * np.pi},
                  "mass_2": {"low": 0., "high": "mass_1"},
                  "mass_1": {"low": 0},
                  "total_mass": {"low": 0.},
                  "chirp_mass": {"low": 0.},
                  "H1_time": {"low": 0.},
                  "L1_time": {"low": 0.},
                  "V1_time": {"low": 0.},
                  "E1_time": {"low": 0.},
                  "spin_1x": {"low": 0., "high": 1.},
                  "spin_1y": {"low": 0., "high": 1.},
                  "spin_1z": {"low": -1., "high": 1.},
                  "spin_2x": {"low": 0., "high": 1.},
                  "spin_2y": {"low": 0., "high": 1.},
                  "spin_2z": {"low": -1., "high": 1.},
                  "chi_p": {"low": 0., "high": 1.},
                  "chi_eff": {"low": -1., "high": 1.},
                  "mass_ratio": {"low": 0., "high": 1.},
                  "symmetric_mass_ratio": {"low": 0., "high": 0.25},
                  "phi_1": {"low": 0., "high": 2 * np.pi},
                  "phi_2": {"low": 0., "high": 2 * np.pi},
                  "cos_tilt_1": {"low": -1., "high": 1.},
                  "cos_tilt_2": {"low": -1., "high": 1.},
                  "redshift": {"low": 0.},
                  "comoving_distance": {"low": 0.},
                  "mass_1_source": {"low": 0.},
                  "mass_2_source": {"low": 0., "high": "mass_1_source"},
                  "chirp_mass_source": {"low": 0.},
                  "total_mass_source": {"low": 0.},
                  "cos_iota": {"low": -1., "high": 1.},
                  "theta_jn": {"low": 0., "high": np.pi},
                  "lambda_1": {"low": 0.},
                  "lambda_2": {"low": 0.},
                  "lambda_tilde": {"low": 0.},
                  "delta_lambda": {}}


class Bounded_2d_kde(kde):
    """Class to generate a two-dimensional KDE for a probability distribution
    functon that exists on a bounded domain
    """
    def __init__(self, pts, xlow=None, xhigh=None, ylow=None, yhigh=None,
                 *args, **kwargs):
        pts = np.atleast_2d(pts)
        assert pts.ndim == 2, 'Bounded_kde can only be two-dimensional'
        super(Bounded_2d_kde, self).__init__(pts.T, *args, **kwargs)
        self._xlow = xlow
        self._xhigh = xhigh
        self._ylow = ylow
        self._yhigh = yhigh

    @property
    def xlow(self):
        """The lower bound of the x domain
        """
        return self._xlow

    @property
    def xhigh(self):
        """The upper bound of the x domain
        """
        return self._xhigh

    @property
    def ylow(self):
        """The lower bound of the y domain
        """
        return self._ylow

    @property
    def yhigh(self):
        """The upper bound of the y domain
        """
        return self._yhigh

    def evaluate(self, pts):
        """Return an estimate of the density evaluated at the given
        points."""
        pts = np.atleast_2d(pts)
        assert pts.ndim == 2, 'points must be two-dimensional'

        x, y = pts.T
        pdf = super(Bounded_2d_kde, self).evaluate(pts.T)
        if self.xlow is not None:
            pdf += super(Bounded_2d_kde, self).evaluate([2 * self.xlow - x, y])

        if self.xhigh is not None:
            pdf += super(Bounded_2d_kde, self).evaluate([2 * self.xhigh - x, y])

        if self.ylow is not None:
            pdf += super(Bounded_2d_kde, self).evaluate([x, 2 * self.ylow - y])

        if self.yhigh is not None:
            pdf += super(Bounded_2d_kde, self).evaluate([x, 2 * self.yhigh - y])

        if self.xlow is not None:
            if self.ylow is not None:
                pdf += super(Bounded_2d_kde, self).evaluate(
                    [2 * self.xlow - x, 2 * self.ylow - y])

            if self.yhigh is not None:
                pdf += super(Bounded_2d_kde, self).evaluate(
                    [2 * self.xlow - x, 2 * self.yhigh - y])

        if self.xhigh is not None:
            if self.ylow is not None:
                pdf += super(Bounded_2d_kde, self).evaluate(
                    [2 * self.xhigh - x, 2 * self.ylow - y])
            if self.yhigh is not None:
                pdf += super(Bounded_2d_kde, self).evaluate(
                    [2 * self.xhigh - x, 2 * self.yhigh - y])
        return pdf

    def __call__(self, pts):
        pts = np.atleast_2d(pts)
        out_of_bounds = np.zeros(pts.shape[0], dtype='bool')

        if self.xlow is not None:
            out_of_bounds[pts[:, 0] < self.xlow] = True
        if self.xhigh is not None:
            out_of_bounds[pts[:, 0] > self.xhigh] = True
        if self.ylow is not None:
            out_of_bounds[pts[:, 1] < self.ylow] = True
        if self.yhigh is not None:
            out_of_bounds[pts[:, 1] > self.yhigh] = True

        results = self.evaluate(pts)
        results[out_of_bounds] = 0.
        return results



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org> This program is free
# software; you can redistribute it and/or modify it under the terms of the GNU
# General Public License as published by the Free Software Foundation; either
# version 3 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

try:
    import pepredicates as pep
    PEP = True
except ImportError:
    PEP = False

import numpy as np
import pandas as pd
from pesummary.utils.utils import logger


def get_classifications(samples):
    """Return the classifications from a dictionary of samples

    Parameters
    ----------
    samples: dict
        dictionary of samples
    """
    from pesummary.utils.utils import RedirectLogger

    default_error = (
        "Failed to generate source classification probabilities because {}"
    )
    try:
        with RedirectLogger("PEPredicates", level="DEBUG") as redirector:
            parameters = list(samples.keys())
            samples = [
                [samples[parameter][j] for parameter in parameters] for j in
                range(len(samples[parameters[0]]))
            ]
            data = PEPredicates.classifications(samples, parameters)
        classifications = {
            "default": data[0], "population": data[1]
        }
    except ImportError:
        logger.warn(
            default_error.format("'PEPredicates' is not installed")
        )
        classifications = None
    except Exception as e:
        logger.warn(default_error.format("%s" % (e)))
        classifications = None
    return classifications


class PEPredicates(object):
    """Class to handle the PEPredicates package
    """
    @staticmethod
    def default_predicates():
        """Set the default possibilities
        """
        default = {
            'BNS': pep.BNS_p,
            'NSBH': pep.NSBH_p,
            'BBH': pep.BBH_p,
            'MassGap': pep.MG_p}
        return default

    @staticmethod
    def check_for_install():
        """Check that predicates is installed
        """
        if not PEP:
            raise ImportError(
                "Failed to import 'predicates' and therefore unable to "
                "calculate astro/terrestrial probabilities")

    @staticmethod
    def convert_to_PEPredicated_data_frame(samples, parameters):
        """Convert the inputs to a pandas data frame compatible with
        PEPredicated

        Parameters
        ----------
        samples: list
            list of samples for a specific result file
        parameters: list
            list of parameters for a specific result file
        """
        PEPredicates.check_for_install()
        psamps = pd.DataFrame()

        mapping = {"mass_1_source": "m1_source",
                   "mass_2_source": "m2_source",
                   "luminosity_distance": "dist",
                   "redshift": "redshift",
                   "a_1": "a1",
                   "a_2": "a2"}

        if not all(i in parameters for i in list(mapping.keys())):
            raise Exception(
                "Failed to generate classification probabilities because not "
                "all required parameters have been provided.")
        for num, i in enumerate(list(mapping.keys())):
            psamps[mapping[i]] = [j[parameters.index(i)] for j in samples]
        return psamps

    @staticmethod
    def resample_to_population(samples):
        """Return samples that have been resampled to a sensibile population

        Parameters
        ----------
        samples: list
            list of samples for a specific result file
        """
        PEPredicates.check_for_install()
        return pep.rewt_approx_massdist_redshift(samples)

    @staticmethod
    def default_classification(samples, parameters):
        """Return the source classification probabilities using the default
        prior used

        Parameters
        ----------
        samples: list
            list of samples for a specific result file
        """
        PEPredicates.check_for_install()
        core_samples = PEPredicates.convert_to_PEPredicated_data_frame(
            samples, parameters)
        ptable = pep.predicate_table(
            PEPredicates.default_predicates(), core_samples)
        for key, value in ptable.items():
            ptable[key] = np.round(value, 5)
        return ptable

    @staticmethod
    def population_classification(samples, parameters):
        """Return the source classification probabilities using a population
        prior

        Parameters
        ----------
        samples: list
            list of samples for a specific result file
        """
        PEPredicates.check_for_install()
        core_samples = PEPredicates.convert_to_PEPredicated_data_frame(
            samples, parameters)
        psamps_resamples = PEPredicates.resample_to_population(core_samples)
        ptable = pep.predicate_table(
            PEPredicates.default_predicates(), psamps_resamples)
        for key, value in ptable.items():
            ptable[key] = np.round(value, 5)
        return ptable

    @staticmethod
    def classifications(samples, parameters):
        """Return the source classification probabilities using both the default
        prior used in the analysis and the population prior
        """
        pop = PEPredicates.population_classification(samples, parameters)
        default = PEPredicates.default_classification(samples, parameters)
        return default, pop

    @staticmethod
    def plot(samples, parameters, population_prior=True):
        """Make a plot of the samples classified by type

        Parameters
        ----------
        samples: list
            list of samples for a specific result file
        """
        logger.debug("Generating the PEPredicates plot")
        PEPredicates.check_for_install()
        core_samples = PEPredicates.convert_to_PEPredicated_data_frame(
            samples, parameters)
        if population_prior:
            psamps_resamples = PEPredicates.resample_to_population(core_samples)
        else:
            psamps_resamples = core_samples
        ptable = {"BBH": pep.is_BBH(psamps_resamples),
                  "BNS": pep.is_BNS(psamps_resamples),
                  "NSBH": pep.is_NSBH(psamps_resamples),
                  "MassGap": pep.is_MG(psamps_resamples)}
        fig = pep.plot_predicates(ptable, psamps_resamples)
        return fig



# Copyright (C) 2019 Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import os
import numpy as np

import pesummary
from pesummary.core.webpage.main import _WebpageGeneration as _CoreWebpageGeneration
from pesummary import conf


class CommandLineCaption(object):
    """Class to handle generating the command line used to generate a plot and
    a caption to describe the plot

    Parameters
    ----------
    """
    def __init__(self, command_line_arguments, samples=None):
        self.args = command_line_arguments
        self.samples = samples
        self.executable = self.get_executable("summaryplots")

    @staticmethod
    def get_executable(executable):
        """Return the path to an executable

        Parameters
        ----------
        executable: str
            the name of the executable you wish to find
        """
        from subprocess import check_output

        path = check_output(["which", executable]).decode("utf-8").strip()
        return path

    @property
    def command_line(self):
        """Generate the command line used to generate the plot
        """
        return "{} {}".format(self.executable, " ".join(self.args))

    @property
    def caption(self):
        """Make a caption to describe the plot
        """
        general_cli = ""
        if "1d_histogram" in self.args[0]:
            return self.histogram_caption()
        if "skymap" in self.args[0]:
            return self.skymap_caption()
        return general_cli

    def histogram_caption(self):
        """Return a caption to describe the 1d histogram plot
        """
        args = self.args[0].split(" ")
        parameter = args[args.index("--parameter") + 1]
        general_cli = (
            "1d histogram showing the posterior distribution for "
            "{}.".format(parameter)
        )
        if parameter == "chi_p":
            general_cli += (
                " chi_p is the precession parameter and quantifies how much "
                "precession there is in the system. It ranges from 0 to 1 "
                "with 0 meaning there is no precession in the system and 1 "
                "meaning there is maximal precession in the system."
            )
        if self.samples is not None:
            general_cli += (
                " The median of the distribution is {} with 90% confidence "
                "interval {}".format(
                    np.round(self.samples.average(type="median"), 3),
                    [np.round(i, 3) for i in self.samples.confidence_interval()]
                )
            )
        return general_cli

    def skymap_caption(self):
        """Return a caption to describe the skymap plot
        """
        general_cli = (
            "Skymap showing the possible location of the source of the "
            "source of the gravitational waves"
        )


class _WebpageGeneration(_CoreWebpageGeneration):
    """
    """
    def __init__(
        self, webdir=None, samples=None, labels=None, publication=None,
        user=None, config=None, same_parameters=None, base_url=None,
        file_versions=None, hdf5=None, colors=None, custom_plotting=None,
        pepredicates_probs=None, gracedb=None, approximant=None, key_data=None,
        file_kwargs=None, existing_labels=None, existing_config=None,
        existing_file_version=None, existing_injection_data=None,
        existing_samples=None, existing_metafile=None, add_to_existing=False,
        existing_file_kwargs=None, result_files=None, notes=None,
        pastro_probs=None
    ):
        self.pepredicates_probs = pepredicates_probs
        self.pastro_probs = pastro_probs
        self.gracedb = gracedb
        self.approximant = approximant
        self.key_data = key_data
        self.file_kwargs = file_kwargs
        self.publication = publication
        self.result_files = result_files

        super(_WebpageGeneration, self).__init__(
            webdir=webdir, samples=samples, labels=labels,
            publication=publication, user=user, config=config,
            same_parameters=same_parameters, base_url=base_url,
            file_versions=file_versions, hdf5=hdf5, colors=colors,
            custom_plotting=custom_plotting,
            existing_labels=existing_labels, existing_config=existing_config,
            existing_file_version=existing_file_version,
            existing_injection_data=existing_injection_data,
            existing_samples=existing_samples,
            existing_metafile=existing_metafile,
            existing_file_kwargs=existing_file_kwargs,
            add_to_existing=add_to_existing, notes=notes
        )

    def categorize_parameters(self, parameters):
        """Categorize the parameters into common headings

        Parameters
        ----------
        parameters: list
            list of parameters that you would like to sort
        """
        params = []
        for heading, category in self.categories.items():
            if any(
                any(j in i for j in category["accept"]) for i in parameters
            ):
                cond = self._condition(category["accept"], category["reject"])
                params.append(
                    [heading, self._partition(cond, parameters)]
                )
        return params

    def _condition(self, true, false):
        """Setup a condition

        Parameters
        ----------
        true: list
            list of strings that you would like to include
        false: list
            list of strings that you would like to neglect
        """
        if len(true) != 0 and len(false) == 0:
            condition = lambda j: True if any(i in j for i in true) else \
                False
        elif len(true) == 0 and len(false) != 0:
            condition = lambda j: True if any(i not in j for i in false) \
                else False
        elif len(true) and len(false) != 0:
            condition = lambda j: True if any(i in j for i in true) and \
                any(i not in j for i in false) else False
        return condition

    def make_navbar_for_result_page(self):
        """Make a navbar for the result page homepage
        """
        links = {
            i: ["1d Histograms", [{"Multiple": i}]] for i in self.labels
        }
        for num, label in enumerate(self.labels):
            for j in self.categorize_parameters(self.samples[label].keys()):
                j = [j[0], [{k: label} for k in j[1]]]
                links[label].append(j)

        final_links = {
            i: [
                "home", ["Result Pages", self._result_page_links()],
                {"Corner": i}, {"Config": i}, links[i]
            ] for i in self.labels
        }
        if len(self.samples) > 1:
            for i in self.labels:
                final_links[i][1][1] += ["Comparison"]
        for num, label in enumerate(self.labels):
            if self.pepredicates_probs[label] is not None:
                final_links[label].append({"Classification": label})
        return final_links

    def generate_webpages(self):
        """Generate all webpages for all result files passed
        """
        if self.add_to_existing:
            self.add_existing_data()
        self.make_home_pages()
        self.make_1d_histogram_pages()
        self.make_corner_pages()
        self.make_config_pages()
        if len(self.labels) > 1:
            self.make_comparison_pages()
        if self.publication:
            self.make_publication_pages()
        if all(val is not None for key, val in self.pepredicates_probs.items()):
            self.make_classification_pages()
        self.make_error_page()
        self.make_version_page()
        self.make_logging_page()
        if self.notes is not None:
            self.make_notes_page()
        self.generate_specific_javascript()

    def _make_home_pages(self, pages):
        """Make the home pages

        Parameters
        ----------
        pages: list
            list of pages that you wish to create
        """
        title = None if self.gracedb is None else (
            "Parameter Estimation Summary Pages for {}".format(self.gracedb)
        )
        banner = "Summary" if self.gracedb is None else (
            "Summary for {}".format(self.gracedb)
        )
        html_file = self.setup_page("home", self.navbar["home"], title=title)
        html_file.make_banner(approximant=banner, key="Summary")
        path = self.image_path["home"]

        if len(self.labels) > 1:
            if not all(self.approximant[i] is not None for i in self.labels):
                image_contents = []
                image_contents.append(
                    os.path.join(path, "compare_time_domain_waveforms.png")
                )
                image_contents = [image_contents]
                html_file.make_table_of_images(contents=image_contents)
                images = [y for x in image_contents for y in x]
                html_file.make_modal_carousel(images=images)

        for i in self.labels:
            html_file.make_banner(approximant=i, key=i)
            image_contents = []
            basic_string = os.path.join(self.webdir, "plots", "{}.png")
            relative_path = os.path.join(path, "{}.png")
            if os.path.isfile(basic_string.format("%s_strain" % (i))):
                image_contents.append(relative_path.format("%s_strain" % (i)))
            if os.path.isfile(basic_string.format("%s_psd_plot" % (i))):
                image_contents.append(relative_path.format("%s_psd_plot" % (i)))
            if os.path.isfile(
                basic_string.format("%s_waveform_time_domain" % (i))
            ):
                image_contents.append(
                    relative_path.format("%s_waveform_time_domain" % (i))
                )
            if os.path.isfile(
                basic_string.format("%s_calibration_plot" % (i))
            ):
                image_contents.append(
                    relative_path.format("%s_calibration_plot" % (i))
                )
            image_contents = [image_contents]
            html_file.make_table_of_images(contents=image_contents)
            images = [y for x in image_contents for y in x]
            html_file.make_modal_carousel(images=images)
            if self.file_kwargs[i]["sampler"] != {}:
                html_file.make_banner(
                    approximant="Sampler kwargs", key="sampler_kwargs",
                    _style="font-size: 26px;"
                )
                _style = "margin-top:3em; margin-bottom:5em;"
                html_file.make_container(style=_style)
                _class = "row justify-content-center"
                html_file.make_div(4, _class=_class, _style=None)
                keys = list(self.file_kwargs[i]["sampler"].keys())
                table_contents = [
                    [self.file_kwargs[i]["sampler"][key] for key in keys]
                ]
                html_file.make_table(
                    headings=keys, format="table-hover", contents=table_contents,
                    heading_span=1, accordian=False
                )
                html_file.end_div(4)
                html_file.end_container()
            if self.file_kwargs[i]["meta_data"] != {}:
                html_file.make_banner(
                    approximant="Meta data", key="meta_data",
                    _style="font-size: 26px; margin-top: -3em"
                )
                _style = "margin-top:3em; margin-bottom:5em;"
                html_file.make_container(style=_style)
                _class = "row justify-content-center"
                html_file.make_div(4, _class=_class, _style=None)
                keys = list(self.file_kwargs[i]["meta_data"].keys())
                table_contents = [
                    [self.file_kwargs[i]["meta_data"][key] for key in keys]
                ]
                html_file.make_table(
                    headings=keys, format="table-hover", contents=table_contents,
                    heading_span=1, accordian=False
                )
                html_file.end_div(4)
                html_file.end_container()
        html_file.close()

        for num, i in enumerate(self.labels):
            html_file = self.setup_page(
                i, self.navbar["result_page"][i], i, approximant=i,
                title="{} Summary page".format(i),
                background_colour=self.colors[num]
            )
            html_file.make_banner(approximant=i, key=i)
            images, cli = self.default_images_for_result_page(i)
            html_file.make_table_of_images(contents=images, cli=cli)
            images = [y for x in images for y in x]
            html_file.make_modal_carousel(images=images)

            if self.custom_plotting:
                from glob import glob

                custom_plots = glob(
                    "{}/plots/{}_custom_plotting_*".format(self.webdir, i)
                )
                path = self.image_path["other"]
                for num, i in enumerate(custom_plots):
                    custom_plots[num] = path + i.split("/")[-1]
                image_contents = [
                    custom_plots[i:4 + i] for i in range(
                        0, len(custom_plots), 4
                    )
                ]
                html_file.make_table_of_images(contents=image_contents)
                images = [y for x in image_contents for y in x]
                html_file.make_modal_carousel(images=images)

            key_data = self.key_data
            contents = []
            for j in self.samples[i].keys():
                row = []
                row.append(j)
                for k in range(len(self.key_data[i])):
                    row.append(np.round(self.key_data[i][j]["maxL"], 3))
                for k in range(len(self.key_data[i])):
                    row.append(np.round(self.key_data[i][j]["mean"], 3))
                for k in range(len(self.key_data[i])):
                    row.append(np.round(self.key_data[i][j]["median"], 3))
                for k in range(len(self.key_data[i])):
                    row.append(np.round(self.key_data[i][j]["std"], 3))
                contents.append(row)

            table_contents = []
            for j in contents:
                one_approx_content = [j[0]] + [
                    j[k * len(self.labels) + num + 1] for k in range(4)
                ]
                table_contents.append(one_approx_content)
            html_file.make_table(
                headings=[" ", "maxL", "mean", "median", "std"],
                contents=table_contents, heading_span=1
            )
            html_file.make_footer(user=self.user, rundir=self.webdir)
            html_file.close()

    def _make_corner_pages(self, pages):
        """Make the corner pages

        Parameters
        ----------
        pages: list
            list of pages that you wish to create
        """
        for num, i in enumerate(self.labels):
            html_file = self.setup_page(
                "{}_Corner".format(i), self.navbar["result_page"][i], i,
                title="{} Corner Plots".format(i), approximant=i,
                background_colour=self.colors[num]
            )
            html_file.make_banner(approximant=i, key="corner")
            params = [
                "luminosity_distance", "dec", "a_2", "phase",
                "a_1", "geocent_time", "phi_jl", "psi", "ra",
                "mass_2", "mass_1", "phi_12", "tilt_2", "iota",
                "tilt_1", "chi_p", "chirp_mass", "mass_ratio",
                "symmetric_mass_ratio", "total_mass", "chi_eff",
                "redshift", "mass_1_source", "mass_2_source",
                "total_mass_source", "chirp_mass_source"
            ]
            included_parameters = [
                i for i in list(self.samples[i].keys()) if i in params
            ]
            html_file.make_search_bar(
                sidebar=included_parameters,
                popular_options=self.popular_options, label=i
            )
            html_file.make_footer(user=self.user, rundir=self.webdir)
            html_file.close()

    def _make_comparison_pages(self, pages):
        """Make the comparison pages

        Parameters
        ----------
        pages: list
            list of pages that you wish to create
        """
        html_file = self.setup_page(
            "Comparison", self.navbar["comparison"],
            title="Comparison Summary Page", approximant="Comparison"
        )
        html_file.make_banner(approximant="Comparison", key="Comparison")
        path = self.image_path["other"]
        base = os.path.join(path, "{}.png")
        contents = [
            [base.format("combined_skymap"), base.format("compare_waveforms")]
        ]
        html_file.make_table_of_images(contents=contents)
        images = [y for x in contents for y in x]
        html_file.make_modal_carousel(images=images)
        if self.custom_plotting:
            from glob import glob

            custom_plots = glob(
                os.path.join(
                    self.webdir, "plots", "combined_custom_plotting_*"
                )
            )
            for num, i in enumerate(custom_plots):
                custom_plots[num] = path + i.split("/")[-1]
            image_contents = [
                custom_plots[i:4 + i] for i in range(0, len(custom_plots), 4)
            ]
            html_file.make_table_of_images(contents=image_contents)
            images = [y for x in image_contents for y in x]
            html_file.make_modal_carousel(images=images)

        if self.comparison_stats is not None:
            rows = range(len(self.labels))
            base = (
                "margin-top:{}em; margin-bottom:{}em; background-color:#FFFFFF; "
                "box-shadow: 0 0 5px grey;"
            )
            style_ks = base.format(5, 1)
            style_js = base.format(0, 5)

            table_contents = {
                i: [
                    [self.labels[j]] + self.comparison_stats[i][0][j] for j in
                    rows
                ] for i in self.same_parameters
            }
            html_file.make_table(
                headings=[" "] + self.labels, contents=table_contents,
                heading_span=1, accordian_header="KS test total", style=style_ks
            )
            table_contents = {
                i: [
                    [self.labels[j]] + self.comparison_stats[i][1][j] for j in
                    rows
                ] for i in self.same_parameters
            }
            html_file.make_table(
                headings=[" "] + self.labels, contents=table_contents,
                heading_span=1, accordian_header="JS test total", style=style_js
            )
        html_file.make_footer(user=self.user, rundir=self.webdir)

        for num, i in enumerate(self.same_parameters):
            html_file = self.setup_page(
                "Comparison_{}".format(i), self.navbar["comparison"],
                title="Comparison PDF for {}".format(i),
                approximant="Comparison"
            )
            html_file.make_banner(approximant="Comparison", key="Comparison")
            path = self.image_path["other"]
            contents = [
                [path + "combined_1d_posterior_{}.png".format(i)],
                [
                    path + "combined_cdf_{}.png".format(i),
                    path + "combined_boxplot_{}.png".format(i)
                ]
            ]
            html_file.make_table_of_images(
                contents=contents, rows=1, columns=2, code="changeimage"
            )
            if self.comparison_stats is not None:
                table_contents = [
                    [self.labels[j]] + self.comparison_stats[i][0][j]
                    for j in rows
                ]
                html_file.make_table(
                    headings=[" "] + self.labels, contents=table_contents,
                    heading_span=1, accordian_header="KS test",
                    style=style_ks
                )
                table_contents = [
                    [self.labels[j]] + self.comparison_stats[i][1][j]
                    for j in rows
                ]
                html_file.make_table(
                    headings=[" "] + self.labels, contents=table_contents,
                    heading_span=1, accordian_header="JS divergence test",
                    style=style_js
                )
            html_file.make_footer(user=self.user, rundir=self.webdir)
            html_file.close()
        html_file = self.setup_page(
            "Comparison_Multiple", self.navbar["comparison"],
            approximant="Comparison", title="Comparison Posteriors for multiple"
        )
        html_file.make_search_bar(
            sidebar=self.same_parameters, label="None", code="combines",
            popular_options=self.popular_options.append(
                {"all": ", ".join(self.same_parameters)}
            )
        )
        html_file.make_footer(user=self.user, rundir=self.webdir)
        html_file.close()

    def make_publication_pages(self):
        """Wrapper function for _make_publication_pages()
        """
        pages = ["Publication"]
        self.create_blank_html_pages(pages)
        self._make_publication_pages(pages)

    def _make_publication_pages(self, pages):
        """Make the publication pages

        Parameters
        ----------
        pages: list
            list of pages that you wish to create
        """
        from glob import glob

        executable = self.get_executable("summarypublication")
        general_cli = "%s --webdir %s --samples %s --labels %s --plot {}" % (
            executable, os.path.join(self.webdir, "plots", "publication"),
            " ".join(self.result_files), " ".join(self.labels)
        )
        html_file = self.setup_page(
            "Publication", self.navbar["home"], title="Publication Plots"
        )
        html_file.make_banner(approximant="Publication", key="Publication")
        path = self.image_path["other"]
        pub_plots = glob(
            os.path.join(self.webdir, "plots", "publication", "*.png")
        )
        for num, i in enumerate(pub_plots):
            shortened_path = i.split("/plots/")[-1]
            pub_plots[num] = path + shortened_path
        cli = []
        for i in pub_plots:
            filename = i.split("/")[-1]
            if "violin_plot" in filename:
                parameter = filename.split("violin_plot_")[-1].split(".png")[0]
                cli.append(
                    general_cli.format("violin") + " --parameters %s" % (
                        parameter
                    )
                )
            elif "spin_disk" in filename:
                cli.append(general_cli.format("spin_disk"))
            elif "2d_contour" in filename:
                parameters = filename.split("2d_contour_plot_")[-1].split(".png")[0]
                cli.append(
                    general_cli.format("2d_contour") + " --parameters %s" % (
                        parameters.replace("_and_", " ")
                    )
                )
        image_contents = [
            pub_plots[i:4 + i] for i in range(0, len(pub_plots), 4)
        ]
        command_lines = [
            cli[i:4 + i] for i in range(0, len(cli), 4)
        ]
        html_file.make_table_of_images(
            contents=image_contents, cli=command_lines
        )
        images = [y for x in image_contents for y in x]
        html_file.make_modal_carousel(images=images)
        html_file.close()

    def make_classification_pages(self):
        """Wrapper function for _make_publication_pages()
        """
        pages = ["{}_{}_Classification".format(i, i) for i in self.labels]
        self.create_blank_html_pages(pages)
        self._make_classification_pages(pages)

    def _make_classification_pages(self, pages):
        """Make the classification pages

        Parameters
        ----------
        pages: list
            list of pages that you wish to create
        """
        executable = self.get_executable("summaryclassification")
        general_cli = "%s --samples {}" % (executable)
        for num, label in enumerate(self.labels):
            html_file = self.setup_page(
                "{}_Classification".format(label),
                self.navbar["result_page"][label], label,
                title="{} Classification".format(label),
                background_colour=self.colors[num], approximant=label
            )
            html_file.make_banner(approximant=label, key="classification")

            if self.pepredicates_probs[label] is not None:
                html_file.make_container()
                _class = "row justify-content-center"
                html_file.make_div(4, _class=_class, _style=None)
                keys = list(self.pepredicates_probs[label]["default"].keys())
                table_contents = [
                    ["{} prior".format(i)] + [
                        self.pepredicates_probs[label][i][j] for j in keys
                    ] for i in ["default", "population"]
                ]
                if self.pastro_probs[label] is not None:
                    keys += ["HasNS"]
                    keys += ["HasRemnant"]
                    table_contents[0].append(self.pastro_probs[label]["HasNS"])
                    table_contents[0].append(
                        self.pastro_probs[label]["HasRemnant"]
                    )
                    table_contents[1].append("-")
                    table_contents[1].append("-")
                html_file.make_table(
                    headings=[" "] + keys, contents=table_contents,
                    heading_span=1, accordian=False
                )
                html_file.make_cli_button(
                    general_cli.format(self.result_files[num])
                )
                html_file.end_div(4)
                html_file.end_container()
            path = self.image_path["other"]
            base = os.path.join(path, "%s_{}_pepredicates{}.png" % (label))
            image_contents = [
                [
                    base.format("default", ""), base.format("default", "_bar"),
                    base.format("population", ""),
                    base.format("population", "_bar")
                ]
            ]
            base = (
                "%s --webdir %s --labels %s --plot {} "
                "--plot_with_{}_prior" % (
                    general_cli.format(self.result_files[num]),
                    os.path.join(self.webdir, "plots"), label
                )
            )
            command_lines = [
                [
                    base.format("mass_1_mass_2", "default"),
                    base.format("bar", "default"),
                    base.format("mass_1_mass_2", "population"),
                    base.format("bar", "population")
                ]
            ]
            html_file.make_table_of_images(contents=image_contents, cli=command_lines)
            images = [y for x in image_contents for y in x]
            html_file.make_modal_carousel(images=images)
            html_file.make_footer(user=self.user, rundir=self.webdir)
            html_file.close()

    def default_images_for_result_page(self, label):
        """Return the default images that will be displayed on the result page
        """
        path = self.image_path["other"]
        base_string = path + "%s_{}.png" % (label)
        image_contents = [
            [
                base_string.format("1d_posterior_mass_1"),
                base_string.format("1d_posterior_mass_2"),
                base_string.format("1d_posterior_a_1"),
                base_string.format("1d_posterior_a_2")
            ],
            [
                base_string.format("skymap"),
                base_string.format("waveform"),
                base_string.format("1d_posterior_iota"),
                base_string.format("1d_posterior_luminosity_distance")
            ]
        ]
        executable = self.get_executable("summaryplots")
        general_cli = (
            "%s --webdir %s --samples %s --burnin %s --plot {} {} "
            "--labels %s" % (
                executable, os.path.join(self.webdir, "plots"),
                self.result_files[self.labels.index(label)], conf.burnin, label
            )
        )
        cli = [
            [
                general_cli.format("1d_histogram", "--parameter mass_1"),
                general_cli.format("1d_histgram", "--parameter mass_2"),
                general_cli.format("1d_histogram", "--parameter a_1"),
                general_cli.format("1d_histogram", "--parameter a_2"),
            ],
            [
                general_cli.format("skymap", ""),
                general_cli.format("waveform", ""),
                general_cli.format("1d_histogram", "--parameter iota"),
                general_cli.format(
                    "1d_histogram", "--parameter luminosity_distance"
                )
            ]
        ]
        return image_contents, cli

    def default_categories(self):
        """Return the default categories
        """
        categories = self.categories = {
            "masses": {
                "accept": ["mass", "q", "symmetric_mass_ratio"],
                "reject": ["source"]
            },
            "inclination": {
                "accept": ["theta", "iota"], "reject": []
            },
            "spins": {
                "accept": ["spin", "chi_p", "chi_eff", "a_1", "a_2"],
                "reject": ["lambda"]
            },
            "spin_angles": {
                "accept": ["phi", "tilt"], "reject": []
            },
            "tidal": {
                "accept": ["lambda"], "reject": []
            },
            "location": {
                "accept": [
                    "ra", "dec", "psi", "luminosity_distance", "redshift",
                    "comoving_distance"
                ],
                "reject": ["mass_ratio"]
            },
            "timings": {
                "accept": ["time"], "reject": []
            },
            "SNR": {
                "accept": ["snr"], "reject": []
            },
            "others": {
                "accept": ["phase", "likelihood", "prior"], "reject": []
            }
        }
        return categories

    def default_popular_options(self):
        """Return a list of popular options
        """
        popular_options = [
            "mass_1, mass_2", "luminosity_distance, iota, ra, dec",
            "iota, phi_12, phi_jl, tilt_1, tilt_2"
        ]
        return popular_options



#! /usr/bin/env python

# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import os
from pesummary.gw.file.read import read as GWRead
from pesummary.core.file.read import read as Read
from pesummary.gw.file.meta_file import _GWMetaFile
from pesummary.core.file.meta_file import _MetaFile
from pesummary.utils.utils import make_dir

__doc__ = """This executable is used to combine multiple PESummary metafiles
into a single PESummry metafile"""


def command_line():
    """Generate an Argument Parser object to control the command line options
    """
    import argparse

    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("-w", "--webdir", dest="webdir",
                        help="make page and plots in DIR", metavar="DIR",
                        default="./")
    parser.add_argument("-s", "--samples", dest="samples", nargs='+',
                        help="Path to PESummary metafile",
                        default=None)
    parser.add_argument("--save_to_hdf5", action="store_true",
                        help="save the meta file in hdf5 format", default=False)
    parser.add_argument("--gw", action="store_true",
                        help="run with the gravitational wave pipeline",
                        default=False)
    return parser


def grab_information_from_file(samples, read_function, required_parameters):
    """Grab the information stored in the PESummary metafile

    Parameters
    ----------
    samples: list
        list of file paths
    read_function: function
        function to read in the PESummary metafle. Should be either
        pesummary.gw.file.read.read or pesummary.core.file.read.read
    required_parameters: list
        list of attributes that you would like to return from the
        PESummary metafile
    """
    full_list = {j: [] for j in required_parameters}
    for idx, i in enumerate(samples):
        mydict = {}
        f = read_function(i)
        mydict = {j: getattr(f, j) for j in required_parameters}
        nsamples = len(mydict["parameters"])
        for j in required_parameters:
            if isinstance(mydict[j], dict):
                full_list[j].append(mydict[j][full_list["labels"][idx]])
            else:
                for num in range(nsamples):
                    if mydict[j] is not None:
                        full_list[j].append(mydict[j][num])
                    else:
                        full_list[j].append(None)

        duplicated = dict(set(
            (x, full_list["labels"].count(x)) for x in
            filter(lambda rec: full_list["labels"].count(rec) > 1,
                   full_list["labels"])))
        if len(duplicated.keys()) >= 1:
            raise Exception("Unable to combine result files because there are "
                            "duplicated labels")
    return full_list


def gw_meta_file(opts):
    """Read in the files with the pesummary.gw.file.read.read function and
    return a pesummary.gw.file.meta_file._GWMetaFile object

    Parameters
    ----------
    opts: argparse.Namespace
        argparse namespace object that contains the passed arguments
    """
    read_function = GWRead
    metafile_function = _GWMetaFile
    required_parameters = [
        "parameters", "samples", "labels", "config", "injection_parameters",
        "input_version", "extra_kwargs", "calibration", "psd", "approximant"]

    data = grab_information_from_file(
        opts.samples, read_function, required_parameters)

    if data["psd"] is None:
        data["psd"] = [None for i in range(data["labels"])]
    if data["calibration"] is None:
        data["calibration"] = [None for i in range(data["labels"])]
    meta_file = _GWMetaFile(data["parameters"], data["samples"],
                            data["labels"], data["config"],
                            data["injection_parameters"],
                            data["input_version"], data["extra_kwargs"],
                            calibration=data["calibration"], psd=data["psd"],
                            approximant=data["approximant"], webdir=opts.webdir)
    return meta_file


def core_meta_file(opts):
    """Read in the files with the pesummary.core.file.read.read function and
    return a pesummary.core.file.meta_file._MetaFile object

    Parameters
    ----------
    opts: argparse.Namespace
        argparse namespace object that contains the passed arguments
    """
    read_function = Read
    metafile_function = _MetaFile
    required_parameters = [
        "parameters", "samples", "labels", "config", "injection_parameters",
        "input_version", "extra_kwargs"]

    data = grab_information_from_file(
        opts.samples, read_function, required_parameters)

    meta_file = _MetaFile(data["parameters"], data["samples"], data["labels"],
                          data["config"], data["injection_parameters"],
                          data["input_version"], data["extra_kwargs"],
                          webdir=opts.webdir)
    return meta_file


def main():
    """Top level interface for `summarycombine_metafiles`
    """
    parser = command_line()
    opts = parser.parse_args()

    if opts.webdir and not os.path.isdir(opts.webdir + "/samples"):
        make_dir(opts.webdir + "/samples")

    if opts.gw:
        meta_file = gw_meta_file(opts)
    else:
        meta_file = core_meta_file(opts)

    meta_file.generate_meta_file_data()
    if not opts.save_to_hdf5:
        meta_file.save_to_json()
    else:
        meta_file.save_to_hdf5()


if __name__ == "__main__":
    main()
